@@ -7,7 +7,8 @@ def find_path ():
     for path in sys.path:
 	(head, tail) = split (path)
         # Forcing python path to be used as /usr/lib instead of /usr/local/lib
-	if tail == 'site-packages' and path.find ('local') == -1:
+	if (tail == 'site-packages' or tail == 'dist-packages') and \
+                path.find ('local') == -1:
 	    base = basename (head)
 	    print path
 	    #print base + '/' + tail
