@@ -1,10 +1,9 @@
-#!/usr/bin/env python
 #############################################################################
 #
 #  Linux Desktop Testing Project http://ldtp.freedesktop.org
 # 
 #  Author:
-#     A. Nagappan <nagappan@gmail.com>
+#     Nagappan Alagappan <nagappan@gmail.com>
 # 
 #  Copyright 2004 - 2007 Novell, Inc.
 # 
@@ -24,3 +23,6 @@
 #  Boston, MA 02110, USA.
 #
 #############################################################################
+
+from checkbox import *
+from pushbutton import *
