@@ -1,12 +1,12 @@
-#!/usr/bin/env python
 #############################################################################
 #
 #  Linux Desktop Testing Project http://ldtp.freedesktop.org
 # 
 #  Author:
-#     A. Nagappan <nagappan@gmail.com>
+#     Nagappan Alagappan <nagappan@gmail.com>
 # 
 #  Copyright 2004 - 2007 Novell, Inc.
+#  Copyright 2008 - 2009 - Nagappan Alagappan
 # 
 #  This library is free software; you can redistribute it and/or
 #  modify it under the terms of the GNU Library General Public
@@ -26,8 +26,8 @@
 #############################################################################
 
 __author__ = "Nagappan A"
-__maintainer__ = "Nagappan A"
-__version__ = "1.3.0"
+# __maintainer__ = "Nagappan A"
+__version__ = "1.4.0"
 
 import re
 import os
@@ -108,7 +108,6 @@ class record:
         '''Iterates over each app on each desktop til it finds one with a
         matching name, which is then returned.'''
         global recorder
-        _old = None
         if name == None:
             return False
         _desktop = self.registry.getDesktop (0)
@@ -197,7 +196,7 @@ class record:
                     i += 1
                     flag = True
                     break
-            if flag == False:
+            if not flag:
                 if _ldtpDebug:
                     print 'flag False'
                 _accessible = self.appmap.getAppHandle (_windowName)
@@ -230,7 +229,7 @@ class record:
                 else:
                     loopedOnce = True
                     i = index
-                if loopedOnce == False:
+                if not loopedOnce:
                     loopedOnce = True
         #Uncomment the below segment of code to dump to FS
         #print 'getObjectName', name
@@ -263,7 +262,7 @@ class record:
                 _child = _desktop.getChildAtIndex (i)
                 if _child.name == name:
                     _desktop.unref ()
-                    return child
+                    return _child
                 _child.unref ()
             _desktop.unref ()
         if _ldtpDebug:
@@ -765,7 +764,8 @@ class record:
                                                                   self.timeElapsed ())
                     self.__pckt.sendpacket (_xml)
                 if _ldtpDebug:
-                    print 'selectmenuitem (\'' + self.__windowName + '\', \'' + self.__menuItem + '\')'
+                    print "selectmenuitem ('%s', '%s')" % (self.__windowName,
+                                                           self.__menuItem)
                 self.__menuItem = None
             self.__comboListThread.set ()
             # Implementation for click release ends
@@ -926,12 +926,14 @@ class record:
             self.__comboBoxText = _objValue
             self.__comboTextType.set ()
             return
-        if event.source and (_role == Accessibility.ROLE_COMBO_BOX or _role == Accessibility.ROLE_TEXT) and \
+        if event.source and (_role == Accessibility.ROLE_COMBO_BOX or \
+                                 _role == Accessibility.ROLE_TEXT) and \
                (str (event.type).startswith ('object:state-changed:focused')): #and event.detail1 == 0:
-            _windowname = self.__windowName
+            _windowName = self.__windowName
             _windowHandle = self.appmap.getWindowHandle (event.source)
             if _windowHandle is not None:
-                _windowName = self.appmap.getWinNameAppmapFormat (_windowHandle.name, _windowHandle.getRole ())
+                _windowName = self.appmap.getWinNameAppmapFormat (_windowHandle.name,
+                                                                  _windowHandle.getRole ())
                 self.__windowName = _windowName
                 _windowHandle.unref ()
             #if self.__textContent != '':
@@ -943,7 +945,8 @@ class record:
             self.__textContent = ''
             if self.__comboBoxName and self.__comboBoxText and self.__comboTextType.isSet ():
                 # FIXME: Should be modified to use comboselect when selected from list item
-                self.settextvalue (_windowName, self.__comboBoxName, self.__comboBoxText, 'text', str (event.type))
+                self.settextvalue (_windowName, self.__comboBoxName,
+                                   self.__comboBoxText, 'text', str (event.type))
                 self.__comboBoxName = None
                 self.__comboBoxText = None
                 self.__comboTextType.clear ()
@@ -969,11 +972,12 @@ class record:
                 str (event.type).startswith ('object:text-changed:insert') or \
                 str (event.type).startswith ('object:text-changed:delete')): #and event.detail1 == 0:
             _windowName = self.__windowName
-            if self.__textType.isSet () == False:
+            if not self.__textType.isSet ():
                 return
             _windowHandle = self.appmap.getWindowHandle (event.source)
             if _windowHandle is not None:
-                _localWindowName = self.appmap.getWinNameAppmapFormat (_windowHandle.name, _windowHandle.getRole ())
+                _localWindowName = self.appmap.getWinNameAppmapFormat (_windowHandle.name,
+                                                                       _windowHandle.getRole ())
                 _windowHandle.unref ()
                 _objName = self.appmap.getObjRelNameAppmapFormat (event.source)
                 if _objName is None:
