@@ -5,9 +5,10 @@
 #  Author:
 #     M Nagashree <mnagashree@novell.com>
 #     Veerapuram Varadhan <v.varadhan@gmail.com>
-#     A. Nagappan <nagappan@gmail.com>
+#     Nagappan Alagappan <nagappan@gmail.com>
 # 
 #  Copyright 2004 - 2006 Novell, Inc.
+#  Copyright 2007 - 2009 Nagappan Alagappan
 # 
 #  This library is free software; you can redistribute it and/or
 #  modify it under the terms of the GNU Library General Public
@@ -26,8 +27,29 @@
 #
 #############################################################################
 
-import sys, commands, os, string, ldtp, time, xml.dom.minidom
-import threading, re, tempfile
+import gc
+import os
+import re
+import sys
+import time
+import wnck
+import string
+import commands
+import tempfile
+import threading
+import xml.dom.minidom
+
+import ldtp
+
+_ldtpDebug = os.getenv ('LDTP_DEBUG')
+
+gtkImport = True
+try:
+    import gtk.gdk
+except ImportError:
+    if _ldtpDebug:
+        print 'pygtk package not installed'
+    gtkImport = False
 
 statGrabMsg = None
 try:
@@ -43,9 +65,9 @@ def tuplelist2list (lst):
         return d
 
 def getFullPath (path):
-    if path[0] == "~":
+    if path [0] == "~":
         path = os.path.expanduser (path)
-    elif path[0] == ".":
+    elif path [0] == ".":
         path = os.path.abspath (path)
 
     return path
@@ -82,49 +104,57 @@ def imagecompare (imgfile1, imgfile2):
         ldtp.log ('Input file does not exist', 'error')
         raise ldtp.LdtpExecutionError ('Input file does not exist')
 
-def winname_to_winid (winname):
-    fp = os.popen ('digwin -c')
-    lines = fp.readlines ()
-    fp.close ()
-    winid = ''
-    for i in lines:
-        if string.find (string.upper (i), string.upper (winname)) != -1:
-            winid = i
-            break
-    if winid == '':
-        ldtp.log (winname + ' - window doesnot exist', 'error')
-        raise ldtp.error, winname + ' - window doesnot exist'
-    winid = string.split (winid, ' ')[0]
-    winid = string.split (winid, '(')[1]
-    winid = string.split (winid, ')')[0]
-    return winid
-
-def imagecapture (winName = None, outFile = None, resolution1 = None, resolution2 = None, x = 0, y = 0):
+def imagecapture (winName = None, outFile = None, resolution1 = None,
+                  resolution2 = None, x = 0, y = 0):
     # winname == None, let us capture the entire screen
     # if output file name is not passed, then a random file name will be generated in
     # /tmp and it will be returned
-    if winName == None:
-        winid = 'root'
-    else:
-        status = commands.getstatusoutput ('which digwin')
-        if status [0] != 0:
-            raise ldtp.LdtpExecutionError ('To capture invidual window, you need to install digwin of LTFX')
-        winid = winname_to_winid (winname)
-        winid = '0x' + winid
+    winid = None
+    if winName:
+        screen = wnck.screen_get_default()
+        # Process pending gtk+ events so that wnck can find out about 
+        # existing windows.
+        while gtk.events_pending():
+            gtk.main_iteration()
+        for window in screen.get_windows():
+            if winName == window.get_name():
+                winid = window.get_xid()
+                break
     if outFile == None:
-        file = tempfile.NamedTemporaryFile ()
-        tmpFile = file.name + '.png'
-        file.close ()
+        fp = tempfile.NamedTemporaryFile ()
+        tmpFile = fp.name + '.png'
+        fp.close ()
     else:
         tmpFile = outFile
-    cmd = "import -window %s" % winid
-    if resolution1 and resolution2:
-        cmd = "%s -crop %dx%d+%d+%d" % (cmd, resolution1, resolution2, x, y)
-    cmd = "%s %s" % (cmd, tmpFile)
-    status = commands.getstatusoutput (cmd)
-    if status [0] != 0:
-        ldtp.log ('Unable to capture', 'error')
-        return None
+    pb = None
+    if gtkImport is True and winid is None:
+        # Based on http://ubuntuforums.org/showthread.php?t=448160
+        window = gtk.gdk.get_default_root_window ()
+        size = window.get_size ()
+        pb = gtk.gdk.Pixbuf (gtk.gdk.COLORSPACE_RGB, False, 8, 
+                             resolution1 or size [0], 
+                             resolution2 or size [1])
+        pb = pb.get_from_drawable (window, window.get_colormap (),
+                                   x, y, 0, 0, 
+                                   resolution1 or size [0], 
+                                   resolution2 or size [1])
+    if pb:
+        pb.save (tmpFile, "png")
+        # based on http://faq.pygtk.org/index.py?req=show&file=faq08.004.htp
+        del pb
+        gc.collect ()
+    else:
+        if  winid == None:
+            winid = 'root'
+        cmd = "import -window %s" % winid
+        if resolution1 and resolution2:
+            cmd = "%s -crop %dx%d+%d+%d" % (cmd, resolution1,
+                                            resolution2, x, y)
+        cmd = "%s %s" % (cmd, tmpFile)
+        status = commands.getstatusoutput (cmd)
+        if status [0] != 0:
+            ldtp.log ('Unable to capture screenshot: %s' % status [1], 'error')
+            return None
     return tmpFile
 
 def blackoutregion (infile, outfile, topx, topy, botx, boty):
@@ -144,14 +174,13 @@ def blackoutregion (infile, outfile, topx, topy, botx, boty):
 class LdtpDataFileParser:
     def __init__ (self, filename = None):
         self.ldtpdataxml = []
-        if filename != None:
-            ldtpDebug = os.getenv ('LDTP_DEBUG')
+        if filename:
             try:
                 dom = xml.dom.minidom.parse (filename)
                 self.ldtpdataxml = dom.getElementsByTagName ("data")
                 if self.ldtpdataxml == []:
                     ldtp.log ('data xml tag not present')
-                    if ldtpDebug != None and ldtpDebug == '2':
+                    if _ldtpDebug and _ldtpDebug == '2':
                         print 'data xml tag not present'
             except xml.parsers.expat.ExpatError, msg:
                 ldtp.log ('XML Error: ' + str (msg), 'error')
@@ -159,21 +188,20 @@ class LdtpDataFileParser:
                 ldtp.log ('XML \"' + filename + '\" file not found', 'error')
     def setfilename (self, filename):
         self.ldtpdataxml = []
-        if filename != None:
-            ldtpDebug = os.getenv ('LDTP_DEBUG')
+        if filename:
             try:
                 dom = xml.dom.minidom.parse (filename)
                 self.ldtpdataxml = dom.getElementsByTagName ("data")
                 if self.ldtpdataxml == []:
                     ldtp.log ('data xml tag not present')
-                    if ldtpDebug != None and ldtpDebug == '2':
+                    if _ldtpDebug and _ldtpDebug == '2':
                         print 'data xml tag not present'
             except xml.parsers.expat.ExpatError, msg:
-                if ldtpDebug != None and ldtpDebug == '2':
+                if _ldtpDebug and _ldtpDebug == '2':
                     print 'XML Error: ' + str (msg)
                 ldtp.log ('XML Error: ' + str (msg), 'error')
             except IOError:
-                if ldtpDebug != None and ldtpDebug == '2':
+                if _ldtpDebug and _ldtpDebug == '2':
                     print 'XML \"' + filename + '\" file not found'
                 ldtp.log ('XML \"' + filename + '\" file not found', 'error')
     def getText (self, nodelist):
