@@ -3,9 +3,9 @@
 #  Linux Desktop Testing Project http://ldtp.freedesktop.org
 # 
 #  Author:
-#      Nagappan A <nagappan@gmail.com>
+#      Nagappan Alagappan <nagappan@gmail.com>
 # 
-#  Copyright 2008 Nagappan Alagappan
+#  Copyright 2008-2009 Nagappan Alagappan
 # 
 #  This library is free software; you can redistribute it and/or
 #  modify it under the terms of the GNU Library General Public
@@ -24,9 +24,9 @@
 #
 #############################################################################
 
-__author__ = "Nagappan A <nagappan@gmail.com>"
-__maintainer__ = "Nagappan A <nagappan@gmail.com>"
-__version__ = "1.0.0"
+__author__ = "Nagappan Alagappan <nagappan@gmail.com>"
+# __maintainer__ = "Nagappan Alagappan <nagappan@gmail.com>"
+__version__ = "1.5.0"
 
 import ldtp
 
@@ -58,12 +58,21 @@ class component:
                 return ''
             raise ldtp.LdtpExecutionError ('Object \'name\' argument required')
 
+    def getwindowsize (self):
+        return ldtp.getwindowsize (self.__window)
+
+    def getobjectsize (self, name):
+        return ldtp.getobjectsize (self.__window, name)
+
     def waittillguiexist (self, name = '', guiTimeOut = None):
         return ldtp.waittillguiexist (self.__window, self.getName (name, False), guiTimeOut)
 
     def waittillguinotexist (self, name = '', guiTimeOut = None):
         return ldtp.waittillguinotexist (self.__window, self.getName (name, False), guiTimeOut)
 
+    def guiexist (self):
+        return ldtp.guiexist (self.__window)
+
     def hasstate (self, name = '', checkState = state.INVALID):
         return ldtp.hasstate (self.__window, self.getName (name), checkState)
 
@@ -385,11 +394,7 @@ class component:
         if length < 1:
             raise ldtp.LdtpExecutionError ('Min 1 argument required')
         name = self.__name
-        if length == 3:
-            return ldtp.rightclick (self.__window, args [0], args [1], args [2])
-        elif length == 2 and name != '':
-            return ldtp.rightclick (self.__window, name, args [0], args [1])
-        elif length == 1 and name != '':
+        if length == 1 and name != '':
             return ldtp.rightclick (self.__window, name, args [0])
         elif length == 2:
             return ldtp.rightclick (self.__window, args [0], args [1])
@@ -503,8 +508,8 @@ class component:
     def verifydropdown (self, name = ''):
         return ldtp.verifydropdown (self.__window, self.getName (name))
 
-    def verfyhidelist (self, name = ''):
-        return ldtp.verfyhidelist (self.__window, self.getName (name))
+    def verifyhidelist (self, name = ''):
+        return ldtp.verifyhidelist (self.__window, self.getName (name))
 
     def verifyselect (self, *args):
         if len (args) < 1:
@@ -541,6 +546,15 @@ class component:
     def getvalue (self, name = ''):
         return ldtp.getvalue (self.__window, self.getName (name))
 
+    def getminvalue (self, name = ''):
+        return ldtp.getminvalue (self.__window, self.getName (name))
+
+    def getmaxvalue (self, name = ''):
+        return ldtp.getmaxvalue (self.__window, self.getName (name))
+
+    def getminincrement (self, name = ''):
+        return ldtp.getminincrement (self.__window, self.getName (name))
+
     def setvalue (self, *args):
         if len (args) < 1:
             raise ldtp.LdtpExecutionError ('Min 1 argument required')
@@ -627,8 +641,8 @@ class component:
     def scrolldown (self, name = ''):
         return ldtp.scrolldown (self.__window, self.getName (name))
 
-    def scrolleft (self, name = ''):
-        return ldtp.scrolleft (self.__window, self.getName (name))
+    def scrollleft (self, name = ''):
+        return ldtp.scrollleft (self.__window, self.getName (name))
 
     def scrollright (self, name = ''):
         return ldtp.scrollright (self.__window, self.getName (name))
@@ -746,13 +760,13 @@ class component:
 
     def doesrowexist (self, *args):
         length = len (args)
-        if length < 2:
-            raise ldtp.LdtpExecutionError ('Min 2 arguments required')
+        if length < 1:
+            raise ldtp.LdtpExecutionError ('Min 1 argument required')
         name = self.__name
-        if length == 3:
-            return ldtp.doesrowexist (self.__window, args [0], args [1], args [2])
-        elif length == 2:
-            return ldtp.doesrowexist (self.__window, name, args [0], args [1])
+        if length == 2:
+            return ldtp.doesrowexist (self.__window, args [0], args [1])
+        elif length == 1:
+            return ldtp.doesrowexist (self.__window, self.getName (''), args [0])
         else:
             raise ldtp.LdtpExecutionError ('Mismatch arguments')
 
@@ -956,8 +970,17 @@ class context (component):
         else:
             return component (self.__window, name)
 
-    def onwidnowcreate (self, cbFunction):
-        return ldtp.onwidnowcreate (self.__window, cbFunction)
+    def onwindowcreate (self, cbFunction):
+        return ldtp.onwindowcreate (self.__window, cbFunction)
+
+    def getwindowsize (self):
+        return ldtp.getwindowsize (self.__window)
+
+    def waittillguiexist (self, name = '', guiTimeOut = None):
+        return ldtp.waittillguiexist (self.__window, self.getName (name, False), guiTimeOut)
+
+    def waittillguinotexist (self, name = '', guiTimeOut = None):
+        return ldtp.waittillguinotexist (self.__window, self.getName (name, False), guiTimeOut)
 
     def guiexist (self):
         return ldtp.guiexist (self.__window)
