@@ -4,9 +4,10 @@
  *
  * Author:
  *    Veerapuram Varadhan <v.varadhan@gmail.com>
- *    Nagappan <nagappan@gmail.com>
+ *    Nagappan Alagappan <nagappan@gmail.com>
  *
  * Copyright 2004 - 2006 Novell, Inc.
+ * Copyright 2007 - 2008 Nagappan Alagappan
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -46,6 +47,7 @@ extern gboolean ldtp_script_service;
 extern gboolean ldtp_external_xml_file;
 
 extern pthread_mutex_t cb_mutex;
+static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
 
 static gboolean
 is_cctxt_registered_callback (gpointer key, gpointer value, gpointer user_data)
@@ -107,7 +109,7 @@ write2file (char *content, long len)
 		g_print ("Unable to create tmp file - mkstemp\n");
 		return NULL;
 	}
-	write (fd, content, len);
+	len = write (fd, content, len);
 	close (fd);
 	return g_strdup (filename);
 }
@@ -138,16 +140,12 @@ generate_response_packet (LDTPClientContext* cctxt, LDTPErrorCode* err, char **r
 		STATUS_END_ELEMENT_LEN +
 		RESPONSE_END_ELEMENT_LEN;
 
-	/* Varadhan
-	   FIXME: Localization?? 
-	*/
-
 	g_print ("resp_len = %d\n", resp_len);
 
 	if (cctxt->req->request_id)
-		resp_len += g_utf8_strlen ((gchar *)cctxt->req->request_id, -1);
+		resp_len += strlen ((gchar *)cctxt->req->request_id);
 	if (cctxt->resp->resp_status)
-		resp_len += g_utf8_strlen (ldtp_error_get_message (cctxt->resp->resp_status), -1);
+		resp_len += strlen (ldtp_error_get_message (cctxt->resp->resp_status));
 
 	memset (buf, 0x00, 16);
 	sprintf (buf, "%d", cctxt->resp->resp_status);
@@ -382,7 +380,9 @@ add_matching_item_to_list (gpointer key, gpointer value, gpointer list)
 	/*
 	  list_item->role == NULL ? then, let us add all of the entries to the list
 	*/
-	if (list_item->role == NULL || g_hash_table_find (value, is_role_matching, list_item->role)) {
+	if (list_item->role == NULL || g_hash_table_find (value,
+							  is_role_matching,
+							  list_item->role)) {
 		label = get_property (value, "label_by", NULL);
 		if (!label)
 			label = get_property (value, "label", NULL);
@@ -664,11 +664,9 @@ has_state (LDTPClientContext* cctxt,
 	for (i = 0; i < len; i++) {
 		state = g_slist_nth_data (cctxt->req->arg_list, i);
 		if (state) { 
-			g_print ("Command: %s\n", state);
 			state_id = atol (state);
-			g_print ("state_id: %ld\n", state_id);
-		}
-		else {
+			g_print ("Command: %s - state_id: %ld\n", state, state_id);
+		} else {
 			*err = LDTP_ERROR_INVALID_STATE;
 			goto error;
 		}
@@ -688,11 +686,71 @@ has_state (LDTPClientContext* cctxt,
 	cctxt->gui_handle = NULL;
 }
 
-static gboolean
-remove_app_list_table (gpointer key, gpointer value, gpointer user_data)
+static void
+get_all_states (LDTPClientContext* cctxt,
+		LDTPErrorCode* err)
 {
-	g_free (key);
-	return TRUE;
+	guint i;
+	char *state = NULL;
+	long int state_id;
+	AccessibleStateSet *state_set;
+
+	state_set = Accessible_getStateSet (cctxt->gui_handle->handle);
+
+	if (!state_set) {
+		*err = LDTP_ERROR_INVALID_STATE;
+		goto error;
+	}
+
+	if (cctxt->resp->data) {
+		g_free (cctxt->resp->data);
+		cctxt->resp->data = NULL;
+	}
+	for (i = SPI_STATE_INVALID; i < SPI_STATE_LAST_DEFINED; i++) {
+		if (AccessibleStateSet_contains (state_set, i)) {
+			if (cctxt->resp->data)
+				cctxt->resp->data = g_strdup_printf ("%s<OBJECT>%d</OBJECT>",
+								     cctxt->resp->data, i);
+			else
+				cctxt->resp->data = g_strdup_printf ("<OBJECT>%d</OBJECT>",
+								     i);
+		}
+	}
+	if (cctxt->resp->data) {
+		char *tmp = NULL;
+		/*
+		  FRAME the output in XML format
+		*/
+		tmp = g_strdup_printf ("%s<OBJECTLIST>%s</OBJECTLIST>",
+				       XML_HEADER, cctxt->resp->data);
+		g_free (cctxt->resp->data);
+		cctxt->resp->data = NULL;
+		if (tmp) {
+			cctxt->resp->data = tmp;
+			cctxt->resp->data_len = strlen (cctxt->resp->data);
+			g_print ("LIST: %s - Data length: %ld\n",
+				 cctxt->resp->data, cctxt->resp->data_len);
+		} else {
+			*err = LDTP_ERROR_UNABLE_TO_GET_OBJECT_STATE;
+			g_print ("%s\n", ldtp_error_get_message (*err));
+			log_msg (LDTP_LOG_CAUSE,
+				 ldtp_error_get_message (*err),
+				 cctxt->log_fp);
+			goto error;
+		}
+	} else {
+		*err = LDTP_ERROR_UNABLE_TO_GET_OBJECT_STATE;
+		g_print ("%s\n", ldtp_error_get_message (*err));
+		log_msg (LDTP_LOG_CAUSE,
+			 ldtp_error_get_message (*err),
+			 cctxt->log_fp);
+		goto error;
+	}
+	*err = LDTP_ERROR_SUCCESS;
+ error:
+	AccessibleStateSet_unref (state_set);
+	ldtp_gui_free_gui_handle (cctxt->gui_handle);
+	cctxt->gui_handle = NULL;
 }
 
 static void
@@ -702,30 +760,18 @@ update_context_name (LDTPClientContext* cctxt)
 		return;
 	if (cctxt->req && cctxt->req->context) {
 		char *window_name = NULL;
-		gboolean flag = FALSE;
-		Accessible *context_handle = get_window_handle (NULL, cctxt->req->context, &window_name, &flag);
+		if (cctxt->app_handle) { // AC: avoiding more require to app level
+			Accessible_unref (cctxt->app_handle);
+			cctxt->app_handle = NULL;
+		}
+		cctxt->app_handle = get_window_handle (NULL, cctxt->app_under_test,
+						       cctxt->req->context,
+						       &window_name, NULL);
 		if (window_name) {
 			g_free (cctxt->req->context);
 			cctxt->req->context = g_strdup (window_name);
 			g_free (window_name);
 		}
-		Accessible_unref (context_handle);
-		/*
-		if (context_handle) {
-			char *name = Accessible_getName (context_handle);
-
-			if (name) {
-				if (g_utf8_collate (name, "")) {
-					g_free (cctxt->req->context);
-					cctxt->req->context = g_strdup (name);
-				}
-				SPI_freeString (name);
-			}
-			Accessible_unref (context_handle);
-		}
-		if (window_name)
-			g_free (window_name);
-		*/
 	}
 }
 
@@ -746,6 +792,10 @@ handle_request (LDTPClientContext* cctxt,
 		return;
 	}
 
+	if (ldtp_req->action_name == NULL) {
+		*err = LDTP_ERROR_INVALID_COMMAND;
+		return;
+	}
 	g_print ("Command: %s\n", ldtp_req->action_name);
 	ldtp_req->command = atol (ldtp_req->action_name);
 	if (ldtp_req->command == LDTP_CMD_INVALID) {
@@ -762,15 +812,33 @@ handle_request (LDTPClientContext* cctxt,
 		return;
 	case LDTP_CMD_LAUNCHAPP: {
 		GError *error = NULL;
-		if (cctxt->req->component && g_ascii_strcasecmp(cctxt->req->component, "1") == 0) {
-			g_setenv ("GTK_MODULES", "gail:atk-bridge", TRUE);
+		gchar *standard_output;
+		gint wait = atoi (cctxt->req->component); //Access
+		gchar *env = g_slist_nth_data (cctxt->req->arg_list, 0); //Access
+		gchar *cml = NULL;
+
+		if (env && g_ascii_strcasecmp(env, "1") == 0) {
+			g_setenv ("GTK_MODULES", "uiaccess:gail:atk-bridge", TRUE);
+			g_setenv ("UIACCESS_MODULES", "alp_maxaccess", TRUE);
 			g_setenv ("GNOME_ACCESSIBILITY", "1", TRUE);
 		}
 		if (cctxt->locale_lang)
 			g_setenv ("LANG", cctxt->locale_lang, TRUE);
-		if (g_spawn_command_line_async (cctxt->req->context, &error)) {
-			// Let us wait so that the application launches
-			sleep (5);
+		//Access
+		if ((cml = g_utf8_strchr(cctxt->req->context, -1, '&'))) {
+			*cml = 0; // AC: FIXME If there are more than one &, it just run the first palce of str
+			if (g_spawn_command_line_async (cctxt->req->context, &error)) {
+				*err = LDTP_ERROR_SUCCESS;
+				if (wait <= 0)
+					wait = 5;
+				sleep (wait);
+			} else
+				*err = LDTP_ERROR_UNABLE_TO_LAUNCH_APP;
+		} else if (g_spawn_command_line_sync (cctxt->req->context,
+						      &standard_output, NULL, NULL, &error)) {
+			sleep (wait);
+			cctxt->resp->data = standard_output;
+			cctxt->resp->data_len = strlen (cctxt->resp->data);
 		}
 		else {
 			if (error)
@@ -896,6 +964,8 @@ handle_request (LDTPClientContext* cctxt,
 	}
 
 	case LDTP_CMD_GENERATEKEYEVENT:
+	case LDTP_CMD_KEYPRESS:
+	case LDTP_CMD_KEYRELEASE:
 		*err = device_main (cctxt, cctxt->req->command);
 		return;
 
@@ -916,16 +986,15 @@ handle_request (LDTPClientContext* cctxt,
 			cctxt->app_handle = NULL;
 		}
 
-		if ((leaked = SPI_exit ())) {
+		if ((leaked = SPI_exit ()))
 			printf ("Leaked %d SPI handles\n", leaked);
-			*err = LDTP_ERROR_UNABLE_TO_REINIT_LDTP;
-			return;
-		}
+
+		sleep (2);
 
 		init_error = SPI_init ();
 
 		if (init_error) {
-			printf ("Error: SPI Init\n");
+			printf ("Error: SPI Init %d\n", init_error);
 			*err = LDTP_ERROR_UNABLE_TO_REINIT_LDTP;
 			return;
 		}
@@ -968,6 +1037,18 @@ handle_request (LDTPClientContext* cctxt,
 		ldtp_gui_wait_till_gui_not_exist (cctxt, err);
 		return;
 
+	case LDTP_CMD_APPUNDERTEST:
+		if (cctxt && cctxt->req && cctxt->req->context) {
+			g_print ("Application under test: %s\n", cctxt->req->context);
+			if (cctxt->app_under_test)
+				g_free (cctxt->app_under_test);
+			cctxt->app_under_test = g_strdup (cctxt->req->context);
+			*err = LDTP_ERROR_SUCCESS;
+			return;
+		}
+		*err = LDTP_ERROR_SET_APP_UNDER_TEST_NAME_FAILED;
+		return;
+
 	case LDTP_CMD_DOESMENUITEMEXIST:
 	case LDTP_CMD_MENUITEMENABLED:
 	case LDTP_CMD_MENUCHECK:
@@ -977,7 +1058,8 @@ handle_request (LDTPClientContext* cctxt,
 	case LDTP_CMD_SELECTMENUITEM:
 		if (g_utf8_strchr (ldtp_req->component, -1, ';')) {
 			gchar **token = NULL;
-			token = g_strsplit (ldtp_req->component, ";", 2); // If 1 is specified to 3rd arg, the result is not correct
+			// If 1 is specified to 3rd arg, the result is not correct
+			token = g_strsplit (ldtp_req->component, ";", 2);
 			gchar *tmp = token [1];
 			cctxt->req->arg_list = g_slist_append (cctxt->req->arg_list, 
 							       g_strdup (tmp));
@@ -1012,40 +1094,52 @@ handle_request (LDTPClientContext* cctxt,
 		bind_text (cctxt, mode, err);
 		return;
 	}
+
+	case LDTP_CMD_GETWINDOWSIZE:
+		ldtp_gui_get_window_size (cctxt, err);
+		return;
+
 	case LDTP_CMD_GETAPPLIST: {
 		char *data = NULL;
 		GHashTable *table;
 		table = get_app_list ();
 		if (table) {
-			g_hash_table_foreach (table, &add_item_to_list, (gpointer)&data);
+			g_hash_table_foreach (table, &add_item_to_list,
+					      (gpointer) &data);
 			cctxt->resp->data = data;
 			if (cctxt->resp->data) {
 				char *tmp = NULL;
 				/*
 				  FRAME the output in XML format
 				*/
-				tmp = g_strdup_printf ("%s<OBJECTLIST>%s</OBJECTLIST>", XML_HEADER, data);
+				tmp = g_strdup_printf ("%s<OBJECTLIST>%s</OBJECTLIST>",
+						       XML_HEADER, data);
 				if (tmp) {
 					g_free (data);
 					cctxt->resp->data = tmp;
-					g_print ("LIST: %s\n", cctxt->resp->data);
-					cctxt->resp->data_len = g_utf8_strlen (cctxt->resp->data, -1);
+					cctxt->resp->data_len = strlen (cctxt->resp->data);
+					g_print ("LIST: %s - Data length: %ld\n",
+						 cctxt->resp->data, cctxt->resp->data_len);
 				} else {
 					*err = LDTP_ERROR_UNABLE_TO_GET_APPLICATION_LIST;
 					g_print ("%s\n", ldtp_error_get_message (*err));
-					log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err), cctxt->log_fp);
+					log_msg (LDTP_LOG_CAUSE,
+						 ldtp_error_get_message (*err),
+						 cctxt->log_fp);
 				}
 			} else {
 				*err = LDTP_ERROR_UNABLE_TO_GET_APPLICATION_LIST;
 				g_print ("%s\n", ldtp_error_get_message (*err));
-				log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err), cctxt->log_fp);
+				log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err),
+					 cctxt->log_fp);
 			}
-			//g_hash_table_foreach_remove (table, remove_app_list_table, NULL);
 			g_hash_table_destroy (table);
 		} else {
 			*err = LDTP_ERROR_UNABLE_TO_GET_APPLICATION_LIST;
-			g_print ("%s - %s\n", __FILE__, ldtp_error_get_message (*err));
-			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err), cctxt->log_fp);
+			g_print ("%s - %s\n", __FILE__,
+				 ldtp_error_get_message (*err));
+			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err),
+				 cctxt->log_fp);
 		}
 		return;
 	}
@@ -1066,7 +1160,7 @@ handle_request (LDTPClientContext* cctxt,
 					g_free (data);
 					cctxt->resp->data = tmp;
 					g_print ("LIST: %s\n", cctxt->resp->data);
-					cctxt->resp->data_len = g_utf8_strlen (cctxt->resp->data, -1);
+					cctxt->resp->data_len = strlen (cctxt->resp->data);
 				} else {
 					*err = LDTP_ERROR_UNABLE_TO_GET_WINDOW_LIST;
 					g_print ("%s\n", ldtp_error_get_message (*err));
@@ -1093,25 +1187,37 @@ handle_request (LDTPClientContext* cctxt,
 		if (!cctxt->req->context) {
 			*err = LDTP_ERROR_ARGUMENT_NULL;
 			g_print ("%s\n", ldtp_error_get_message (*err));
-			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err), cctxt->log_fp);
+			log_msg (LDTP_LOG_CAUSE,
+				 ldtp_error_get_message (*err),
+				 cctxt->log_fp);
 			return;
 		}
 		if (!cctxt->app_map) {
                         /* if appmap not already initialised, do it now */
                         accessible = ldtp_gui_get_gui_handle (cctxt, err);
-                        ldtp_gui_free_gui_handle (accessible);
-                        if (*err != LDTP_ERROR_SUCCESS) {
-				log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err), cctxt->log_fp);
+                        // ldtp_gui_free_gui_handle (accessible);
+			/*
+			  Maybe it got something wrong in ldtp_gui_get_gui_handle,
+			  but let us retry to getinfo
+			  if (*err != LDTP_ERROR_SUCCESS) {
+				log_msg (LDTP_LOG_CAUSE,
+					 ldtp_error_get_message (*err),
+					 cctxt->log_fp);
                                 return;
-                        }
+                          }
+			*/
 		}
-		ht = get_object_def (cctxt->app_map, (char *)cctxt->req->context, NULL, cctxt->log_fp, TRUE);
+		if (cctxt->app_map)
+			ht = get_object_def (cctxt->app_map, (char *)cctxt->req->context,
+					     NULL, cctxt->log_fp, TRUE);
 		if (!ht) {
 			update_cur_window_appmap_handle (cctxt,
 							 err);
-			ht = get_object_def (cctxt->app_map, (char *)cctxt->req->context, NULL, cctxt->log_fp, TRUE);
+			ht = get_object_def (cctxt->app_map, (char *)cctxt->req->context,
+					     NULL, cctxt->log_fp, TRUE);
 			if (*err != LDTP_ERROR_SUCCESS) {
-				g_print ("Unable to update context: %s in appmap", (char *)cctxt->req->context);
+				g_print ("Unable to update context: %s in appmap",
+					 (char *)cctxt->req->context);
 				return;
 			}
 		}
@@ -1170,7 +1276,8 @@ handle_request (LDTPClientContext* cctxt,
 		if (!cctxt->app_map) {
                         /* if appmap not already initialised, do it now */
                         accessible = ldtp_gui_get_gui_handle (cctxt, err);
-                        ldtp_gui_free_gui_handle (accessible);
+			if (accessible->handle != cctxt->app_handle)
+				ldtp_gui_free_gui_handle (accessible);
                         if (*err != LDTP_ERROR_SUCCESS) {
 				log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err), cctxt->log_fp);
 				if (name)
@@ -1203,6 +1310,8 @@ handle_request (LDTPClientContext* cctxt,
 		list.data = data;
 		list.name = name == NULL ? NULL : g_utf8_collate (name,
 								  cctxt->req->context) == 0 ? NULL : name;
+		if (list.name)
+			g_print ("List.Name: %s\n", list.name);
 		list.role = role;
 		g_hash_table_foreach (context_ht, &add_matching_item_to_list, (gpointer) &list);
 		cctxt->resp->data = list.data;
@@ -1238,7 +1347,10 @@ handle_request (LDTPClientContext* cctxt,
 		GHashTable *context_ht = NULL;
 		GHashTable *component_ht = NULL;
 
-		update_context_name (cctxt);
+		// update_context_name (cctxt);
+		// AC: if compoent existed previously, this will return a fake info.
+		update_cur_window_appmap_handle (cctxt, err);
+
 		if (!cctxt->req->context || !cctxt->req->component) {
 			*err = LDTP_ERROR_ARGUMENT_NULL;
 			g_print ("%s\n", ldtp_error_get_message (*err));
@@ -1248,7 +1360,8 @@ handle_request (LDTPClientContext* cctxt,
 		if (!cctxt->app_map) {
                         /* if appmap not already initialised, do it now */
                         accessible = ldtp_gui_get_gui_handle (cctxt, err);
-                        ldtp_gui_free_gui_handle (accessible);
+			if (accessible->handle != cctxt->app_handle)
+				ldtp_gui_free_gui_handle (accessible);
                         if (*err != LDTP_ERROR_SUCCESS) {
 				log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err), cctxt->log_fp);
                                 return;
@@ -1300,7 +1413,7 @@ handle_request (LDTPClientContext* cctxt,
 				g_free (data);
 				cctxt->resp->data = tmp;
 				g_print ("LIST: %s\n", cctxt->resp->data);
-				cctxt->resp->data_len = g_utf8_strlen (cctxt->resp->data, -1);
+				cctxt->resp->data_len = strlen (cctxt->resp->data);
 			} else {
 				*err = LDTP_ERROR_UNABLE_TO_GET_OBJECT_LIST;
 				g_print ("%s\n", ldtp_error_get_message (*err));
@@ -1330,7 +1443,8 @@ handle_request (LDTPClientContext* cctxt,
 		if (!cctxt->app_map) {
                         /* if appmap not already initialised, do it now */
                         accessible = ldtp_gui_get_gui_handle (cctxt, err);
-                        ldtp_gui_free_gui_handle (accessible);
+			if (accessible->handle != cctxt->app_handle)
+				ldtp_gui_free_gui_handle (accessible);
                         if (*err != LDTP_ERROR_SUCCESS) {
 				log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (*err), cctxt->log_fp);
                                 return;
@@ -1381,7 +1495,71 @@ handle_request (LDTPClientContext* cctxt,
 			return;
 		}
 		cctxt->resp->data = g_strdup (obj_property);
-		cctxt->resp->data_len = g_utf8_strlen (obj_property, -1);
+		cctxt->resp->data_len = strlen (obj_property);
+		return;
+	}
+	case LDTP_CMD_GENERATEMD5: {
+#ifdef ENABLE_ACCESS
+		FILE *fp;
+		int len = 0;
+		guchar *buffer = 0;
+		gchar *md5 = 0;
+
+		fp = fopen (cctxt->req->context, "rb");
+		if (fp == NULL)
+			return;
+		fseek (fp, 0L, SEEK_END);
+		len = ftell (fp);
+		buffer = (guchar *) malloc (len * sizeof (guchar));
+		if (!buffer)
+			return;
+		rewind (fp);
+		// Just to avoid compilation warning assigned fread
+		// return value to cctxt->resp->data_len
+		cctxt->resp->data_len = fread (buffer, 1, len , fp);
+		fclose (fp);
+
+		md5 = g_compute_checksum_for_data (G_CHECKSUM_MD5, buffer, len);
+		g_print ("MD5 checksum of file: %s is: %s\n", cctxt->req->context, md5);
+		g_free (buffer);
+
+		cctxt->resp->data = md5;
+		cctxt->resp->data_len = strlen (cctxt->resp->data);
+#else
+		*err = LDTP_ERROR_COMMAND_NOT_SUPPORTED;
+#endif
+		return;
+	}
+	case LDTP_CMD_GETFILE: {
+#ifdef ENABLE_ACCESS
+		FILE *fp;
+		int len = 0;
+		guchar *buffer = 0;
+		gchar *encoded = 0;
+
+		fp = fopen (cctxt->req->context, "rb");
+		if (fp == NULL)
+			return;
+		fseek (fp, 0L, SEEK_END);
+		len = ftell (fp);
+		buffer = (guchar *) malloc (len * sizeof (guchar));
+		if (!buffer)
+			return;
+		rewind (fp);
+		// Just to avoid compilation warning assigned fread
+		// return value to cctxt->resp->data_len
+		cctxt->resp->data_len = fread (buffer, 1, len , fp);
+		fclose (fp);
+	
+		encoded = g_base64_encode (buffer, len);
+
+		cctxt->resp->data = encoded;
+		cctxt->resp->data_len = strlen (cctxt->resp->data);
+
+		g_free (buffer);
+#else
+		*err = LDTP_ERROR_COMMAND_NOT_SUPPORTED;
+#endif
 		return;
 	}
 	}
@@ -1394,14 +1572,17 @@ handle_request (LDTPClientContext* cctxt,
 	   arguments.
 	*/
 
+	pthread_mutex_lock (&mutex);
 	accessible = ldtp_gui_get_gui_handle (cctxt, err);
 	if (*err != LDTP_ERROR_SUCCESS) {
 		/* FIXME: Error handling */
 		g_print ("Unable to get handle\n");
+		pthread_mutex_unlock (&mutex);
 		return;
 	} else {
-		char *name;
-		name = Accessible_getRoleName (accessible->handle);
+		char *name = NULL;
+		if (accessible && accessible->handle)
+			name = Accessible_getRoleName (accessible->handle);
 		if (name) {
 			g_print ("Got gui handle - %s\n", name);
 			SPI_freeString (name);
@@ -1413,8 +1594,15 @@ handle_request (LDTPClientContext* cctxt,
 
 	if (cctxt->req->command == LDTP_CMD_HASSTATE) {
 		has_state (cctxt, err);
+		pthread_mutex_unlock (&mutex);
 		return;
 	}
+	if (cctxt->req->command == LDTP_CMD_GETALLSTATES) {
+		get_all_states (cctxt, err);
+		pthread_mutex_unlock (&mutex);
+		return;
+	}
+	pthread_mutex_unlock (&mutex);
 
 	//sleep (1);
 	switch (cctxt->gui_handle->class_id) {
@@ -1534,6 +1722,25 @@ handle_request (LDTPClientContext* cctxt,
 	case SPI_ROLE_TOOL_BAR:
 		*err = tool_bar_main (cctxt, cctxt->req->command);
 		break;
+
+#ifdef ENABLE_ACCESS
+	case SPI_ROLE_TITLE_BAR:
+		*err = title_bar_main (cctxt, cctxt->req->command,
+				       (gchar *)ldtp_req->context);
+		break;
+
+	case SPI_ROLE_REPEAT_BUTTON:
+		*err = repeat_button_main (cctxt, cctxt->req->command);
+		break;
+
+	case SPI_ROLE_COMMANDBAR_BUTTON:
+		*err = commandbar_button_main (cctxt, cctxt->req->command);
+		break;
+
+	case SPI_ROLE_DATE_SELECTOR:
+		*err = date_selector_main (cctxt, cctxt->req->command);
+		break;
+#endif
 	default:
 		if (cctxt->req->command == LDTP_CMD_KBDENTER) {
 			*err = device_main (cctxt, cctxt->req->command);
@@ -1562,13 +1769,12 @@ handle_request (LDTPClientContext* cctxt,
 	   For this, we have to have a switch case on command-id, because
 	   not all commands return data.
 	*/
-	ldtp_gui_free_gui_handle (cctxt->gui_handle);
 }
 
 void *
 handle_client (void *ptr)
 {
-	int sock_fd = ptr;
+	int sock_fd = (int) ptr;
 	char* packet = NULL;
 	char* tmpptr = NULL;
 	uint32_t packet_len;
@@ -1579,12 +1785,14 @@ handle_client (void *ptr)
 	uint32_t resp_size = 0;
 	char *resp_pckt = NULL;
 
-	LDTPErrorCode status;
+	LDTPErrorCode status = 0;
 
 	LDTPClientContext *cctxt = NULL;
 	Packet *pckt = NULL;
 
 	cctxt = g_new0 (LDTPClientContext, 1);
+	if (!cctxt) // Access
+		return NULL;
 	cctxt->app_map = NULL;
 	cctxt->app_handle = NULL;
 	cctxt->req = g_new0 (LDTPRequest, 1);
@@ -1613,6 +1821,8 @@ handle_client (void *ptr)
 		}
 
 		packet = g_malloc0 (packet_len + 1);
+		if (!packet) // ACCESS
+			goto error;
 		tmpptr = packet;
 
 		pckt_len = packet_len;
@@ -1628,31 +1838,42 @@ handle_client (void *ptr)
 				  ldtp_log ("Connection: closed [%s:%d]\n", 
 				  client->ip_address, client->socketfd);
 				*/
-				g_print ("Client data read error\n");
-				log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (status), cctxt->log_fp);
+				g_print ("Client closed connection\n");
+				log_msg (LDTP_LOG_CAUSE,
+					 ldtp_error_get_message (status),
+					 cctxt->log_fp);
 				goto error;
 			}
 
-			if (packet)
-				g_print ("Data read %d, packet-len = %d, bytes read = %ld, data: %s\n", 
+			if (packet && ldtp_debug)
+				g_print ("Data read %d, packet-len = %d, bytes read = %d, data: %s\n", 
 					 pckt_len, packet_len, bytes_read, packet);
 			packet += bytes_read;
 			packet_read += bytes_read;
 			pckt_len = packet_len - packet_read;
-			g_print ("PACKET LENGTH: %d\n", pckt_len);
+			g_print ("Packet length remaing to read: %d\n", pckt_len);
 			if (pckt_len <= 0 || bytes_read == 0)
 				break;
 		}
 		packet = tmpptr;
 
 		if (packet)
-			g_print ("Received packet [%s] through %d\n", packet, sock_fd);
+			g_print ("Received packet [%s] through %d of len: %d\n",
+				 packet, sock_fd, packet_len);
 
 		pckt = g_new0 (Packet, 1);
+		if (!pckt)
+			goto error;
 		pckt->packet = packet;
 		pckt->len = packet_len;
 
 		handle_request (cctxt, pckt, &status);
+		ldtp_gui_free_gui_handle (cctxt->gui_handle);
+		cctxt->gui_handle = NULL;
+		if (cctxt->app_handle) {
+			Accessible_unref (cctxt->app_handle);
+			cctxt->app_handle = NULL;
+		}
 		if (status == LDTP_ERROR_STOP_SCRIPT_ENGINE) {
 			goto error;
 		}
@@ -1708,6 +1929,10 @@ handle_client (void *ptr)
 		g_free (cctxt->window_name);
 		cctxt->window_name = NULL;
 	}
+	if (cctxt->app_under_test) {
+		g_free (cctxt->app_under_test);
+		cctxt->app_under_test = NULL;
+	}
 
 	if (status != LDTP_ERROR_CLIENT_DISCONNECTED) {
 		generate_response_packet (cctxt, &status, &resp_pckt, &resp_size);
@@ -1727,6 +1952,7 @@ handle_client (void *ptr)
 
 	pthread_mutex_lock (&cb_mutex);
 	unregister_window_creation_event (cctxt, &status);
+	pthread_mutex_unlock (&cb_mutex);
 
 	if (cctxt && cctxt->req)
 		ldtp_request_free (cctxt->req, 0);
@@ -1737,6 +1963,8 @@ handle_client (void *ptr)
 
 	if (pckt)
 		g_free (pckt);
+	ldtp_gui_free_gui_handle (cctxt->gui_handle);
+	cctxt->gui_handle = NULL;
 	if (cctxt->app_handle) {
 		/*	
 		  FIXME: If the server closes connection, before the client closes connection,
@@ -1746,12 +1974,13 @@ handle_client (void *ptr)
 		cctxt->app_handle = NULL;
 	}
 
+	pthread_mutex_lock (&cb_mutex);
 	if (client_context) {
 		guint count = g_hash_table_foreach_remove (client_context, is_cctxt_registered_callback, cctxt);
 		g_print ("Removed %d entries from client context hash table\n", count);
 	}
+	pthread_mutex_unlock (&cb_mutex);
 
-	//ldtp_appmap_free (cctxt->app_map);
 	if (cctxt && cctxt->resp)
 		status = cctxt->resp->resp_status;
 	else
@@ -1764,7 +1993,6 @@ handle_client (void *ptr)
 	g_free (cctxt->locale_lang);
 	g_free (cctxt);
 	cctxt = NULL;
-	pthread_mutex_unlock (&cb_mutex);
 
 	if (status == LDTP_ERROR_STOP_SCRIPT_ENGINE) {
 		cleanup (0);
