@@ -4,10 +4,11 @@
  *
  * Author:
  *    Poornima Nayak<pnayak@novell.com>
- *    Nagappan A <nagappan@gmail.com>
+ *    Nagappan Alagappan <nagappan@gmail.com>
  *    Veerapuram Varadhan <v.varadhan@gmail.com>
  *
  * Copyright 2004 - 2006 Novell, Inc.
+ * Copyright 2007 - 2008 Nagappan Alagappan
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -792,6 +793,13 @@ combo_box_main (LDTPClientContext* cctxt, int command)
         case LDTP_CMD_KBDENTER:
                 error = device_main (cctxt, command);
                 break;
+	case LDTP_CMD_GETOBJECTSIZE:
+		cctxt->resp->data_len = 0;
+		cctxt->resp->data = get_size (cctxt->gui_handle->handle, &error);
+		if (cctxt->resp->data) {
+			cctxt->resp->data_len = g_utf8_strlen (cctxt->resp->data, -1);
+		}
+		break;
 	default:
 		error = LDTP_ERROR_COMMAND_NOT_IMPLEMENTED;
 	}
