@@ -302,6 +302,60 @@ get_keyval_id (gchar *input_string, struct KeyValue *keyvals)
 	return LDTP_ERROR_SUCCESS;
 }
 
+static LDTPErrorCode
+press_key (char *input_string, FILE *log_fp)
+{
+	LDTPErrorCode error;
+	struct KeyValue keyval[MAX_TOKENS+1];
+	gint index;
+	SPIBoolean flag;
+
+	error = get_keyval_id (input_string,  keyval);
+	if (error != LDTP_ERROR_SUCCESS) {
+		goto error;
+	}
+
+	for (index=0; keyval[index].value != UNDEFINED_KEY && index < MAX_TOKENS;index++) {
+		g_print ("Generating Key Board Value: %d\n", keyval [index].value);
+		flag = SPI_generateKeyboardEvent (keyval [index].value, NULL, SPI_KEY_PRESS);
+		if (flag == FALSE) {
+			error = LDTP_ERROR_UNABLE_TO_ENTER_KEY;
+			goto error;
+		}
+	}
+	return error;
+ error:
+	log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
+	return error;
+}
+
+static LDTPErrorCode
+release_key (char *input_string, FILE *log_fp)
+{
+	LDTPErrorCode error;
+	struct KeyValue keyval[MAX_TOKENS+1];
+	gint index;
+	SPIBoolean flag;
+
+	error = get_keyval_id (input_string,  keyval);
+	if (error != LDTP_ERROR_SUCCESS) {
+		goto error;
+	}
+
+	for (index=0; keyval[index].value != UNDEFINED_KEY && index < MAX_TOKENS;index++) {
+		g_print ("Generating Key Board Value: %d\n", keyval [index].value);
+		flag = SPI_generateKeyboardEvent (keyval [index].value, NULL, SPI_KEY_RELEASE);
+		if (flag == FALSE) {
+			error = LDTP_ERROR_UNABLE_TO_ENTER_KEY;
+			goto error;
+		}
+	}
+	return error;
+ error:
+	log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
+	return error;
+}
+
 static LDTPErrorCode
 generate_keyboard_sequence (char *input_string, FILE *log_fp)
 {
@@ -345,8 +399,8 @@ generate_keyboard_sequence (char *input_string, FILE *log_fp)
 			while (index--) {
 				if (keyval[index].non_print_key == FALSE)
 					break;
-				g_print ("Releasing: %d\n",keyval[index].value);
-				flag = SPI_generateKeyboardEvent (keyval[index].value, NULL, type);
+				g_print ("Releasing: %d\n", keyval [index].value);
+				flag = SPI_generateKeyboardEvent (keyval [index].value, NULL, type);
 				if (flag == FALSE) {
 					error = LDTP_ERROR_UNABLE_TO_ENTER_KEY;
 					goto error;
@@ -359,8 +413,8 @@ generate_keyboard_sequence (char *input_string, FILE *log_fp)
 	while (index--) {
 		if (keyval[index].non_print_key == FALSE)
 			break;
-		g_print ("Releasing: %d\n",keyval[index].value);
-		flag = SPI_generateKeyboardEvent (keyval[index].value,NULL,type);
+		g_print ("Releasing: %d\n", keyval [index].value);
+		flag = SPI_generateKeyboardEvent (keyval [index].value, NULL, type);
 		if (flag == FALSE) {
 			error = LDTP_ERROR_UNABLE_TO_ENTER_KEY;
 			goto error;
@@ -395,6 +449,12 @@ device_main (LDTPClientContext* cctxt, int command)
 	case LDTP_CMD_GENERATEKEYEVENT:
 		error = generate_keyboard_sequence (cctxt->req->context, cctxt->log_fp);
 		break;
+	case LDTP_CMD_KEYPRESS:
+		error = press_key (cctxt->req->context, cctxt->log_fp);
+		break;
+	case LDTP_CMD_KEYRELEASE:
+		error = release_key (cctxt->req->context, cctxt->log_fp);
+		break;
 	default:
 		error = LDTP_ERROR_COMMAND_NOT_IMPLEMENTED;
 	}
