@@ -3,7 +3,7 @@
  * Linux Desktop Testing Project http://ldtp.freedesktop.org
  *
  * Author:
- *    Nagappan A <nagappan@gmail.com>
+ *    Nagappan Alagappan <nagappan@gmail.com>
  *    S Thanikachalam <sthanikachalam@novell.com>
  *    Premkumar J <jpremkumar@novell.com>
  *
@@ -230,42 +230,6 @@ remove_remapped_entry (gpointer key, gpointer value, gpointer user_data)
 	return TRUE;
 }
 
-/*
-gboolean
-remove_context_entries (gpointer key, gpointer value, gpointer context)
-{
-	GHashTable *component = NULL;
-
-	if (context == NULL || key == NULL)
-		return TRUE;
-	component = g_hash_table_lookup (context, (gchar *)key);
-	if (component) {
-		g_hash_table_destroy (component);
-		g_hash_table_remove (context, component);
-		component = NULL;
-	}
-	return TRUE;
-}
-*/
-
-static gboolean
-remove_appmap_entries (gpointer key, gpointer value, gpointer user_data)
-{
-	GHashTable *context = NULL;
-	GHashTable *appmap = user_data;
-
-	if (!appmap || !key)
-		return TRUE;
-	context = g_hash_table_lookup (appmap, (gchar *)key);
-	if (context) {
-		//g_hash_table_foreach_remove (context, (GHRFunc)&remove_context_entries, context);
-		g_print ("Context: %s %s\n", (gchar *)key, context);
-		g_hash_table_destroy (context);
-		context = NULL;
-	}
-	return TRUE;
-}
-
 /*
   Free application map resource
 */
@@ -276,7 +240,6 @@ ldtp_appmap_free (GHashTable *appmap)
 		return;
 
 	pthread_mutex_lock (&mutex);
-	//g_hash_table_foreach_remove (appmap, (GHRFunc)&remove_appmap_entries, appmap);
 	g_hash_table_destroy (appmap);
 	pthread_mutex_unlock (&mutex);
 	appmap = NULL;
@@ -308,10 +271,10 @@ search_key_based (gpointer key, gpointer value, gpointer user_data)
 static void
 search_window_key_based (gpointer key, gpointer value, gpointer user_data)
 {
-	if (search_key_based (key, value, user_data)) {
-		OBJInfo *obj;
-		obj = (OBJInfo *) user_data;
-		g_hash_table_insert (obj->ht_data_list, g_strdup ((gchar *) key), value);
+	OBJInfo *obj;
+	obj = (OBJInfo *) user_data;
+	if (obj && obj->ht_data_list && search_key_based (key, value, user_data)) {
+		g_hash_table_insert (obj->ht_data_list, g_strdup ((gchar *) key), NULL);
 	}
 }
 
@@ -436,9 +399,9 @@ search_key_glob_based (gpointer key, gpointer value, gpointer user_data)
 static void
 search_window_key_glob_based (gpointer key, gpointer value, gpointer user_data)
 {
-	if (search_key_glob_based (key, value, user_data)) {
-		OBJInfo *obj;
-		obj = (OBJInfo *) user_data;
+	OBJInfo *obj;
+	obj = (OBJInfo *) user_data;
+	if (obj && obj->ht_data_list && search_key_glob_based (key, value, user_data)) {
 		g_hash_table_insert (obj->ht_data_list, g_strdup ((gchar *) key), value);
 	}
 }
@@ -551,9 +514,9 @@ search_label_based (gpointer key, gpointer value, gpointer user_data)
 static void
 search_window_label_based (gpointer key, gpointer value, gpointer user_data)
 {
-	if (search_label_based (key, value, user_data)) {
-		OBJInfo *obj;
-		obj = (OBJInfo *) user_data;
+	OBJInfo *obj;
+	obj = (OBJInfo *) user_data;
+	if (obj && obj->ht_data_list && search_label_based (key, value, user_data)) {
 		g_hash_table_insert (obj->ht_data_list, g_strdup ((gchar *) key), value);
 	}
 }
@@ -618,9 +581,9 @@ search_label_glob_based (gpointer key, gpointer value, gpointer user_data)
 static void
 search_window_label_glob_based (gpointer key, gpointer value, gpointer user_data)
 {
-	if (search_label_glob_based (key, value, user_data)) {
-		OBJInfo *obj;
-		obj = (OBJInfo *) user_data;
+	OBJInfo *obj;
+	obj = (OBJInfo *) user_data;
+	if (obj && obj->ht_data_list && search_label_glob_based (key, value, user_data)) {
 		g_hash_table_insert (obj->ht_data_list, g_strdup ((gchar *) key), value);
 	}
 }
@@ -702,9 +665,8 @@ search_window_after_stripping_space (gpointer key, gpointer value, gpointer user
 	if (!key || !user_data || !value)
 		return;
 	obj = (OBJInfo *) user_data;
-	if (search_obj_after_stripping_space (key, value, obj)) {
-		if (obj && obj->ht_data_list && key && value)
-			g_hash_table_insert (obj->ht_data_list, g_strdup ((gchar *) key), value);
+	if (obj && obj->ht_data_list && search_obj_after_stripping_space (key, value, obj)) {
+		g_hash_table_insert (obj->ht_data_list, g_strdup ((gchar *) key), value);
 	}
 }
 
@@ -733,6 +695,7 @@ get_all_matching_def (GHashTable *ht, gchar *context, gboolean obj_is_window, FI
 	obj.key = context;
 	obj.parent_name = NULL;
 	obj.obj_is_window = obj_is_window;
+	obj.ht_data_list = ht_data_list;
 	g_hash_table_foreach (ht, search_window_key_based, &obj);
 
 	if (!g_hash_table_size (ht_data_list)) {
@@ -761,6 +724,7 @@ get_all_matching_def (GHashTable *ht, gchar *context, gboolean obj_is_window, FI
 		obj.key = context;
 		obj.parent_name = NULL;
 		obj.obj_is_window = obj_is_window;
+		obj.ht_data_list = ht_data_list;
 		g_hash_table_foreach (ht, search_window_label_based, &obj);
 	}
 	if (!g_hash_table_size (ht_data_list)) {
@@ -778,6 +742,7 @@ get_all_matching_def (GHashTable *ht, gchar *context, gboolean obj_is_window, FI
 		obj.key = context;
 		obj.parent_name = NULL;
 		obj.obj_is_window = obj_is_window;
+		obj.ht_data_list = ht_data_list;
 		g_hash_table_foreach (ht, search_window_label_glob_based, &obj);
 		if (pattern)
 			g_pattern_spec_free (pattern);
@@ -790,6 +755,7 @@ get_all_matching_def (GHashTable *ht, gchar *context, gboolean obj_is_window, FI
 		obj.key = context;
 		obj.parent_name = NULL;
 		obj.obj_is_window = obj_is_window;
+		obj.ht_data_list = ht_data_list;
 		g_hash_table_foreach (ht, search_window_after_stripping_space, &obj);
 	}
 	return ht_data_list;
@@ -972,7 +938,7 @@ find_property (gpointer key, gpointer value, gpointer user_data)
 		return FALSE;
 	if (ldtp_debug) {
 		g_print ("User data: %s\n", (gchar *) user_data);
-		//g_print ("Value: %s\n", (gchar *) value);
+		// g_print ("Value: %s\n", (gchar *) value);
 		g_print ("Key: %s\n", (gchar *) key);
 	}
 
