@@ -128,6 +128,9 @@ ldtp_error_get_message (LDTPErrorCode error)
 	case LDTP_ERROR_COMMAND_NOT_IMPLEMENTED:
 		msg = "Requested action on mentioned object is not implemented - Welcome contribution ;)";
 		break;
+	case LDTP_ERROR_COMMAND_NOT_SUPPORTED:
+		msg = "Requested command on mentioned object is not supported on this platform";
+		break;
 	case LDTP_ERROR_GETTEXTVALUE_FAILED:
 		msg = "GetTextValue action on mentioned object failed";
 		break;
@@ -218,7 +221,7 @@ ldtp_error_get_message (LDTPErrorCode error)
 		msg = "Object is not of type component";
 		break;
 	case LDTP_ERROR_INVALID_DATE:
-		msg = "Ivalid date, can't be selected";
+		msg = "Invalid date, can't be selected";
 		break;
 	case LDTP_ERROR_INVALID_OBJECT_STATE:
 		msg = "Invalid object state";
@@ -582,6 +585,16 @@ ldtp_error_get_message (LDTPErrorCode error)
 		break;
 	case LDTP_ERROR_UNABLE_TO_VERIFY_PAGE_TAB_NAME:
 		msg = "Unable to verify page tab name";
+		break;
+	case LDTP_ERROR_UNABLE_TO_SET_SLIDER_VALUE:
+		msg = "Unable to set slider value";
+		break;
+	case LDTP_ERROR_UNABLE_TO_GET_OBJECT_STATE:
+		msg = "Unable to get all object states";
+		break;
+	case LDTP_ERROR_SET_APP_UNDER_TEST_NAME_FAILED:
+		msg = "Setting application name under test failed";
+		break;
 	default:
 		msg = "Error code not found";
 		break;
