@@ -57,6 +57,7 @@ typedef enum _LDTPErrorCode {
 	LDTP_ERROR_UNABLE_TO_GET_CELL_HANDLE_FAILED,
 	LDTP_ERROR_OBJ_INFO_MISMATCH,
 	LDTP_ERROR_COMMAND_NOT_IMPLEMENTED,
+	LDTP_ERROR_COMMAND_NOT_SUPPORTED,
 	LDTP_ERROR_GETTEXTVALUE_FAILED,
 	LDTP_ERROR_STATUSBAR_GETTEXT_FAILED,
 	LDTP_ERROR_STATUSBAR_NOT_VISIBLE,
@@ -209,7 +210,10 @@ typedef enum _LDTPErrorCode {
 	LDTP_ERROR_UNABLE_TO_GET_PAGE_TAB_NAME,
 	LDTP_ERROR_UNABLE_TO_VERIFY_PAGE_TAB_NAME,
 	LDTP_ERROR_CHECK_BOX_STATE_NOT_ENABLED,
-	LDTP_ERROR_UNABLE_TO_GET_WINDOW_LIST
+	LDTP_ERROR_UNABLE_TO_GET_WINDOW_LIST,
+	LDTP_ERROR_UNABLE_TO_SET_SLIDER_VALUE,
+	LDTP_ERROR_UNABLE_TO_GET_OBJECT_STATE,
+	LDTP_ERROR_SET_APP_UNDER_TEST_NAME_FAILED
 } LDTPErrorCode;
 
 const char* ldtp_error_get_message (LDTPErrorCode error);
