@@ -4,8 +4,10 @@
  *
  * Author:
  *    Veerapuram Varadhan <v.varadhan@gmail.com>
+ *    Nagappan Alagappan <nagappan@gmail.com>
  *
  * Copyright 2004 - 2006 Novell, Inc.
+ * Copyright 2007 - 2008 Nagappan Alagappan
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -113,14 +115,12 @@ report_window_event  (const AccessibleEvent *event, void *user_data)
 				SPI_freeString (context);
 			return;
 		}
-		pthread_mutex_lock (&cb_mutex);
 		if ((cctxt = is_window_reg_for_events (title))) {
 			/*
 			  Notify to client
 			*/
 			if (!cctxt || !cctxt->resp) {
 				g_print ("CCTXT is lost\n");
-				pthread_mutex_unlock (&cb_mutex);
 				return;
 			}
 			cctxt->resp->data = g_strdup (title);
@@ -130,7 +130,6 @@ report_window_event  (const AccessibleEvent *event, void *user_data)
 			generate_notification_packet (cctxt, &status, &resp_pckt, &resp_size);
 			if (status != LDTP_ERROR_SUCCESS) {
 				g_print ("Error generating notification\n");
-				pthread_mutex_unlock (&cb_mutex);
 				return;
 			}
 			if (resp_pckt)
@@ -138,7 +137,6 @@ report_window_event  (const AccessibleEvent *event, void *user_data)
 			send_response (cctxt->sock_fd, resp_pckt, resp_size, &status);
 			g_free (cctxt->resp->data);
 			cctxt->resp->data = NULL;
-			pthread_mutex_unlock (&cb_mutex);
 			return;
 		}
 		/*
@@ -156,40 +154,73 @@ report_window_event  (const AccessibleEvent *event, void *user_data)
 			generate_notification_packet (cctxt, &status, &resp_pckt, &resp_size);
 			if (status != LDTP_ERROR_SUCCESS) {
 				g_print ("Error generating notification\n");
-				pthread_mutex_unlock (&cb_mutex);
 				return;
 			}
 
 			g_print ("Notification: %s - Len - %d", resp_pckt, resp_size);
 			send_response (cctxt->sock_fd, resp_pckt, resp_size, &status);
-			pthread_mutex_unlock (&cb_mutex);
 			return;
 		}
 		if (window_name)
 			g_free (window_name);
-		pthread_mutex_unlock (&cb_mutex);
 		SPI_freeString (context);
 		SPI_freeString (title);
 	} else if (g_ascii_strcasecmp ("window:close", event->type) == 0 ||
 		   g_ascii_strcasecmp ("window:deactivate", event->type) == 0 ||
 		   g_ascii_strcasecmp ("window:destroy", event->type) == 0) {
 		LDTPClientContext *cctxt = NULL;
-		char *title = AccessibleWindowEvent_getTitleString (event);
-		char *window_name;
+		gchar *window_name = NULL;
+		gchar *title = AccessibleWindowEvent_getTitleString (event);
 		if (!title) {
 			g_print ("Window title NULL\n");
 			return;
 		}
-		window_name = get_window_name_in_appmap_format (title, Accessible_getRole (event->source));
+		if (!g_utf8_collate (title, "")) {
+			// FIXME: AC if window did not have name, then we remove all this kinda window now
+			long loop = 0;
+			AccessibleRole role = Accessible_getRole (event->source);
+
+			pthread_mutex_lock (&cb_mutex);
+			for ( ; ; ) {
+				gchar *tmp = g_strdup_printf ("%ld", loop++);
+				if (tmp) {
+					window_name = get_window_name_in_appmap_format (tmp,
+											role);
+					g_free (tmp);
+					tmp = NULL;
+				} else
+					break;
+
+				if (window_name) {
+					cctxt = is_window_in_cctxt (window_name);
+					if (cctxt && cctxt->app_map) {
+						g_hash_table_remove (cctxt->app_map,
+								     window_name);
+						g_free (window_name);
+					} else {
+						g_free (window_name);
+						break; // Hash table is indexed by 0
+					}
+				} else {
+					g_print ("BUG: FIXME PLEASE, still unable to get name %s %d\n", __FILE__, __LINE__);
+				}
+			}
+			pthread_mutex_unlock (&cb_mutex);
+			SPI_freeString (title);
+			return;
+		}
+		window_name = get_window_name_in_appmap_format (title,
+								Accessible_getRole (event->source));
 		if (window_name) {
 			g_print ("Window name: %s - %s\n", window_name, title);
-			pthread_mutex_lock (&cb_mutex);
 			cctxt = is_window_in_cctxt (window_name);
-			if (cctxt && cctxt->app_map)
-				g_hash_table_remove (cctxt->app_map, window_name);
-			pthread_mutex_unlock (&cb_mutex);
+			if (cctxt && cctxt->app_map) {
+				if (pthread_mutex_trylock (&cb_mutex) != EBUSY) {
+					g_hash_table_remove (cctxt->app_map, window_name);
+					pthread_mutex_unlock (&cb_mutex);
+				}
+			}
 			g_free (window_name);
-			window_name = NULL;
 		}
 		SPI_freeString (title);
 	}
@@ -249,14 +280,13 @@ cleanup (int mysig)
 	SPI_event_quit ();
 }
 
-static void 
+static int
 accept_connection (int listener, LDTPErrorCode* err)
 {
 	struct sockaddr_in remoteaddr;
 	socklen_t addrlen;
 	int newfd;
 	pthread_t client_thread = 0;
-	int* newfd_dup = NULL;
 	int retval;
   
 	/* handle new connections */
@@ -267,13 +297,11 @@ accept_connection (int listener, LDTPErrorCode* err)
 	if (newfd == -1) {
 		ldtp_log("ERROR:accept() failed with \"%s\"\n", strerror(errno));
 		*err = LDTP_ERROR_ACCEPT_FAILED;
-		goto error;
+		return -1;
 	}
 
 	ldtp_log ("Client connection: accepted\n");
 
-	//newfd_dup = malloc (sizeof (newfd));
-	//*newfd_dup = newfd;
 	retval = pthread_create (&client_thread, NULL, &handle_client, 
 				 (void *) newfd);
 	if (retval == EAGAIN) {
@@ -282,7 +310,7 @@ accept_connection (int listener, LDTPErrorCode* err)
 			  "Disconnecting the client.\n", __FILE__, __LINE__);
 		close_connection (newfd);
 		*err = LDTP_ERROR_THREAD_CREATION_FAILED;
-		goto error;
+		return -1;
 	}
 	else {
 		gchar fd [15];
@@ -292,13 +320,8 @@ accept_connection (int listener, LDTPErrorCode* err)
 		g_hash_table_insert (client_thread_pool, g_strdup (fd), NULL);
 		pthread_mutex_unlock (&client_thread_pool_mutex);
 		*err = LDTP_ERROR_SUCCESS;
-		return;
+		return newfd;
 	}
-
- error:
-	if (newfd_dup)
-		free (newfd_dup);
-	return;
 }
 
 static gboolean
@@ -354,7 +377,6 @@ ldtp_server_thread (void *ptr)
 {
 	struct pollfd pfd;
 	int server_socket;
-	int client_update_flag;
 	int time_out = -1;
 	LDTPErrorCode err;
 
@@ -378,23 +400,22 @@ ldtp_server_thread (void *ptr)
 		if (poll (&pfd, 1, time_out) == -1) {
 			if (errno == EINTR) {
 				continue;
-			}
-			else if (errno != EBADF && errno != EFAULT && errno != EINVAL && errno != ENOMEM) {
+			} else if (errno != EBADF && errno != EFAULT && errno != EINVAL && errno != ENOMEM) {
 				ldtp_log ("%d:Continuing\n", getpid ());
 				exit (1);
-			}
-			else {
+			} else {
 				ldtp_log ("%d:Exiting : %s\n", getpid (), strerror (errno));
 				exit (1);
 			}
 		}
-		client_update_flag = 0;
 		if ((pfd.revents & POLLIN) && (pfd.fd == server_socket)) {
-			accept_connection (server_socket, &err);
+			int new_client_fd = accept_connection (server_socket, &err);
 			if (err != LDTP_ERROR_SUCCESS)
 				ldtp_log ("******Unable to accept connection*******\n");
 			continue;
-		} // if check for all fds
+		}
+		// On time out, check for all fds, if no client is
+		// connected, quit LDTP
 		if (client_thread_pool) {
 			guint size;
 			pthread_mutex_lock (&client_thread_pool_mutex);
