@@ -207,8 +207,7 @@ list_main (LDTPClientContext* cctxt, int command)
 				      count, cctxt->log_fp);
 		break;
 	default:
-		error = ( (LDTP_ERROR_COMMAND_NOT_IMPLEMENTED));
-		break;
+		error = LDTP_ERROR_COMMAND_NOT_IMPLEMENTED;
 	}
 	return error;
 }
