@@ -3,9 +3,10 @@
  * Linux Desktop Testing Project http://ldtp.freedesktop.org
  *
  * Author:
- *    Nagappan A <nagappan@gmail.com>
+ *    Nagappan Alagappan <nagappan@gmail.com>
  *
  * Copyright 2004 - 2006 Novell, Inc.
+ * Copyright 2007 - 2008 Nagappan Alagappan
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -89,7 +90,8 @@ select_menu_item (Accessible *object, FILE *log_fp)
 		SPI_freeString(name);
 		SPI_freeString(desc);
 	}
-	ldtp_nsleep (0, 15000);
+	// AC: if the popuped menu disappera soon, then change it to large
+	ldtp_nsleep (1, 15000);
 	flag = AccessibleAction_doAction (action, 0);
 	Accessible_unref (action);
 	if (flag)
@@ -157,10 +159,23 @@ menu_item_main (LDTPClientContext* cctxt, int command)
 	AccessibleRole class_id;
 
 	switch (command) {
+	case LDTP_CMD_GETOBJECTSIZE:
+		cctxt->resp->data_len = 0;
+		cctxt->resp->data = get_size (cctxt->gui_handle->handle, &error);
+		if (cctxt->resp->data) {
+			cctxt->resp->data_len = g_utf8_strlen (cctxt->resp->data, -1);
+		}
+		break;
 	case LDTP_CMD_CLICK:
 	case LDTP_CMD_SELECTMENUITEM:
 		error = select_menu_item (cctxt->gui_handle->handle, cctxt->log_fp);
 		break;
+	case LDTP_CMD_DOESMENUITEMEXIST:
+		if (cctxt && cctxt->gui_handle && cctxt->gui_handle->handle)
+			error = LDTP_ERROR_SUCCESS;
+		else
+			error = LDTP_ERROR_MENU_ITEM_DOES_NOT_EXIST;
+		break;
 	case LDTP_CMD_MENUITEMENABLED:
 		error = is_menu_item_enabled (cctxt->gui_handle->handle, cctxt->log_fp);
 		break;
