@@ -68,43 +68,62 @@ select_menu_item (Accessible *object, char *param)
 	return LDTP_ERROR_SUCCESS;
 }
 
+static LDTPErrorCode
+is_menu_item_enabled (Accessible *object, FILE *log_fp)
+{
+	SPIBoolean enabled;
+	AccessibleStateSet *state;
+
+	state = Accessible_getStateSet (object);
+	enabled = AccessibleStateSet_contains (state, SPI_STATE_SENSITIVE);
+	if (enabled)
+		return LDTP_ERROR_SUCCESS;
+	else {
+		LDTPErrorCode error;
+		error = LDTP_ERROR_MENU_ITEM_STATE_DISABLED;
+		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
+		return error;
+	}
+}
+
 static LDTPErrorCode
 list_child_menu_items (Accessible *object, char **data, FILE *log_fp)
 {
 	long i;
 	long child_count;
-	char *name = NULL;
+	char *name;
 	Accessible *child;
-	LDTPErrorCode error;
 
 	child_count = Accessible_getChildCount (object);
-	if (child_count > 0) {
-		for (i = 0; i < child_count; i++) {
-			child = Accessible_getChildAtIndex (object, i);
-			if (child) {
-				name = Accessible_getName (child);
-				if (name && g_ascii_strcasecmp (name, "") != 0) {
-					if (*data == NULL)
-						*data = g_strdup (name);
-					else {
-						char *tmp;
-						tmp = g_strdup_printf ("%s;%s", *data, name);
-						g_free (*data);
-						*data = NULL;
-						*data = tmp;
-					}
-					g_print ("Child menu item: %s", name);
-					log_msg (LDTP_LOG_INFO, name, log_fp);
-					SPI_freeString (name);
-				}
-				Accessible_unref (child);
-			}
-		}
-	} else {
-		error = LDTP_ERROR_MENU_ITEM_DOES_NOT_EXIST;
+	if (child_count <= 0) {
+		LDTPErrorCode error = LDTP_ERROR_MENU_ITEM_DOES_NOT_EXIST;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
+	for (i = 0; i < child_count; i++) {
+		child = Accessible_getChildAtIndex (object, i);
+		if (!child)
+			continue;
+		name = Accessible_getName (child);
+		if (name) {
+			if (g_ascii_strcasecmp (name, "") == 0) {
+				SPI_freeString (name);
+				continue;
+			}
+			if (*data == NULL)
+				*data = g_strdup (name);
+			else {
+				char *tmp;
+				tmp = g_strdup_printf ("%s;%s", *data, name);
+				g_free (*data);
+				*data = tmp;
+			}
+			g_print ("Child menu item: %s", name);
+			log_msg (LDTP_LOG_INFO, name, log_fp);
+			SPI_freeString (name);
+		}
+		Accessible_unref (child);
+	}
 	return LDTP_ERROR_SUCCESS;
 }
 
@@ -168,6 +187,7 @@ menu_main (LDTPClientContext* cctxt, int command, char *window_name)
 {
 	LDTPErrorCode error;
 	char msg [256];
+	gchar *role_name;
 
 	switch (command) {
 	case LDTP_CMD_SELECTMENUITEM:
@@ -182,17 +202,25 @@ menu_main (LDTPClientContext* cctxt, int command, char *window_name)
 		LDTPGuiHandle *accessible = NULL;
 		Accessible *object = cctxt->gui_handle->handle;
 
+		// AC: BUG FIX, if incorrect value of selectmenuitem ('dlg0', 'mnu0')
+		if (!cctxt->req->arg_list)
+			return LDTP_ERROR_ARGUMENT_NULL;
+
 		// Invoke menu
-		select_menu_item (cctxt->gui_handle->handle, cctxt->req->arg_list->data);
+		select_menu_item (cctxt->gui_handle->handle,
+				  cctxt->req->arg_list->data);
 
 		if (cctxt && cctxt->req && cctxt->req->component)
-			g_print ("DEBUG: %s - %d - %s\n", __FILE__, __LINE__, cctxt->req->component);
+			g_print ("DEBUG: %s - %d - %s\n", __FILE__, __LINE__,
+				 cctxt->req->component);
 
 		ldtp_req = cctxt->req;
 		//params = mnuFile;mnuNew
-		if (cctxt->req && cctxt->req->arg_list && cctxt->req->arg_list->data) {
+		if (cctxt->req && cctxt->req->arg_list &&
+		    cctxt->req->arg_list->data) {
 			gchar **token = NULL;
-			token = g_strsplit (ldtp_req->arg_list->data, ";", 2); // If 1 is specified to 3rd arg, the result is not correct
+			// If 1 is specified to 3rd arg, the result is not correct
+			token = g_strsplit (ldtp_req->arg_list->data, ";", 2);
 			gchar *tmp = token [1];
 			ldtp_req->arg_list = g_slist_remove_all (ldtp_req->arg_list,
 								 ldtp_req->arg_list->data);
@@ -230,8 +258,16 @@ menu_main (LDTPClientContext* cctxt, int command, char *window_name)
 			return LDTP_ERROR_UNABLE_TO_GET_CHILD_MENU_ITEM;
 		}
 		class = Accessible_getRole (accessible->handle);
+		role_name = Accessible_getRoleName (accessible->handle);
+		if (role_name) {
+			g_print ("Role: %s\n", role_name);
+			SPI_freeString (role_name);
+		}
 		cctxt->gui_handle->handle = accessible->handle;
-		if (class == SPI_ROLE_MENU)
+		if (ldtp_req->arg_list && g_slist_length (ldtp_req->arg_list) == 0 &&
+		    class == SPI_ROLE_MENU && command == LDTP_CMD_MENUITEMENABLED)
+			error = is_menu_item_enabled (cctxt->gui_handle->handle, cctxt->log_fp);
+		else if (class == SPI_ROLE_MENU)
 			error = menu_main (cctxt, command, window_name);
 		else if ((command == LDTP_CMD_DOESMENUITEMEXIST) && class == SPI_ROLE_MENU_ITEM) {
 			if (accessible)
