@@ -3,9 +3,10 @@
  * Linux Desktop Testing Project http://ldtp.freedesktop.org
  *
  * Author:
- *    Nagappan A <nagappan@gmail.com>
+ *    Nagappan Alagappan <nagappan@gmail.com>
  *
  * Copyright 2004 - 2006 Novell, Inc.
+ * Copyright 2007 - 2008 Nagappan Alagappan
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -254,10 +255,17 @@ page_tab_list_main (LDTPClientContext* cctxt, int command)
 			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 			break;
 		}
-		g_print ("Tab name: %ld\n", cctxt->req->arg_list->data);
+		g_print ("Tab name: %s\n", (char *)cctxt->req->arg_list->data);
 		error = verify_tab_name (cctxt->gui_handle->handle,
 					 cctxt->req->arg_list->data, cctxt->log_fp);
 		break;
+	case LDTP_CMD_GETOBJECTSIZE:
+		cctxt->resp->data_len = 0;
+		cctxt->resp->data = get_size (cctxt->gui_handle->handle, &error);
+		if (cctxt->resp->data) {
+			cctxt->resp->data_len = g_utf8_strlen (cctxt->resp->data, -1);
+		}
+		break;
 	default:
 		error = LDTP_ERROR_COMMAND_NOT_IMPLEMENTED;
 	}
