@@ -53,6 +53,9 @@ static int table = 0,
 	tree_table = 0,
 	layered_pane = 0,
 	text = 0,
+	auto_complete = 0,
+	calendar = 0,
+	cal_event = 0,
 	cal_view = 0,
 	panel = 0,
 	filler = 0,
@@ -72,7 +75,9 @@ static int table = 0,
 	unknown = 0,
 	embedded_component = 0,
 	dialog = 0,
-	frame = 0;
+	frame = 0,
+	icon = 0,
+	tool_tip = 0;
 
 static void
 reset_count()
@@ -91,6 +96,9 @@ reset_count()
 	tree_table = 0;
 	layered_pane = 0;
 	text = 0;
+	auto_complete = 0;
+	calendar = 0;
+	cal_event = 0;
 	cal_view = 0;
 	panel = 0;
 	filler = 0;
@@ -112,6 +120,8 @@ reset_count()
 	embedded_component = 0;
 	dialog = 0;
 	frame = 0;
+	icon = 0;
+	tool_tip = 0;
 }
 
 gchar*
@@ -346,7 +356,7 @@ filter_appmap_data (Accessible *accessible, OBJECT_INFO *obj_info, char *label)
 	    (g_ascii_strcasecmp (obj_info->object_type, "separator") == 0 ||
 	     g_ascii_strcasecmp (obj_info->object_type, "table_cell") == 0))
 		return 0;
-	if (label && g_ascii_strcasecmp (label, "ukngrip") == 0)
+	if (label && g_ascii_strcasecmp (label, "unkgrip") == 0)
 		return 0;
 	if (obj_info->object_type && g_ascii_strcasecmp (obj_info->object_type, "menu_item") == 0) {
 		Accessible *parent = NULL;
@@ -465,14 +475,19 @@ get_object_info (Accessible *accessible)
 		layered_pane++;
 		obj_info->object_type = g_strdup ("layered_pane");
 	}
+	else if (role == SPI_ROLE_ICON) {
+		obj_info->prefix = g_strdup ("ico");
+		obj_info->instance_index = icon++;
+		obj_info->object_type = g_strdup ("icon");
+	}
 	else if (role == SPI_ROLE_FRAME) {
 		obj_info->prefix = g_strdup ("frm");
-		obj_info->instance_index = frame++; /* Value -1 signifies not applicable */
+		obj_info->instance_index = frame++;
 		obj_info->object_type = g_strdup ("frame");
 	}
 	else if (role == SPI_ROLE_DIALOG) {
 		obj_info->prefix = g_strdup ("dlg");
-		obj_info->instance_index = dialog++; /* Value -1 signifies not applicable */
+		obj_info->instance_index = dialog++;
 		obj_info->object_type = g_strdup ("dialog");
 	}
 	else if (role == SPI_ROLE_WINDOW) {
@@ -482,19 +497,19 @@ get_object_info (Accessible *accessible)
 	}
 	else if (role == SPI_ROLE_FILE_CHOOSER) {
 		obj_info->prefix = g_strdup ("dlg");
-		obj_info->instance_index = dialog++; /* Value -1 signifies not applicable */
+		obj_info->instance_index = dialog++;
 		obj_info->object_type = g_strdup ("file_chooser");
 	}
 	else if (role == SPI_ROLE_ALERT) {
 		obj_info->prefix = g_strdup ("dlg");
-		obj_info->instance_index = dialog++; /* Value -1 signifies not applicable */
+		obj_info->instance_index = dialog++;
 		obj_info->object_type = g_strdup ("alert");
 	}
 	else if (role == SPI_ROLE_CALENDAR) {
-		obj_info->prefix = g_strdup ("calview");
-		obj_info->instance_index = cal_view;
-		cal_view++;
-		obj_info->object_type = g_strdup ("calendar_view");
+		obj_info->prefix = g_strdup ("cal");
+		obj_info->instance_index = calendar;
+		calendar++;
+		obj_info->object_type = g_strdup ("calendar");
 	}
 	else if (role == SPI_ROLE_PANEL) {
 		obj_info->prefix = g_strdup ("pnl");
@@ -587,15 +602,13 @@ get_object_info (Accessible *accessible)
 		text++;
 		obj_info->object_type = g_strdup ("entry");
 	}
-#endif
-	/*
 	else if (role == SPI_ROLE_AUTOCOMPLETE) {
-		obj_info->prefix = g_strdup ("txt");
-		obj_info->instance_index = text;
-		text++;
+		obj_info->prefix = g_strdup ("auto");
+		obj_info->instance_index = auto_complete;
+		auto_complete++;
 		obj_info->object_type = g_strdup ("auto_complete");
 	}
-	*/
+#endif
 	else if (role == SPI_ROLE_PARAGRAPH) {
 		obj_info->prefix = g_strdup ("txt");
 		obj_info->instance_index = text;
@@ -674,6 +687,12 @@ get_object_info (Accessible *accessible)
 		tool_bar++;
 		obj_info->object_type = g_strdup ("tool_bar");
 	}
+	else if (role == SPI_ROLE_TOOL_TIP) {
+		obj_info->prefix = g_strdup ("ttip");
+		obj_info->instance_index = tool_tip;
+		tool_tip++;
+		obj_info->object_type = g_strdup ("tool_tip");
+	}
 	else if (role == SPI_ROLE_CHECK_BOX) {
 		obj_info->prefix = g_strdup ("chk");
 		obj_info->instance_index = check_box;
@@ -696,19 +715,19 @@ get_object_info (Accessible *accessible)
 		name = Accessible_getRoleName (accessible);
 		if (name) {
 			if (g_utf8_collate (name, "Calendar View") == 0) {
-				obj_info->prefix = g_strdup ("cal");
+				obj_info->prefix = g_strdup ("calview");
 				obj_info->instance_index = cal_view;
 				obj_info->object_type = g_strdup ("calendar_view");
 				cal_view++;
 			}
 			else if (g_utf8_collate (name, "Calendar Event") == 0) {
-				obj_info->prefix = g_strdup ("cal");
-				obj_info->instance_index = cal_view;
+				obj_info->prefix = g_strdup ("calevent");
+				obj_info->instance_index = cal_event;
 				obj_info->object_type = g_strdup ("calendar_event");
-				cal_view++;
+				cal_event++;
 			}
 			else {
-				obj_info->prefix = g_strdup ("ukn");
+				obj_info->prefix = g_strdup ("unk");
 				obj_info->instance_index = unknown;
 				unknown++;
 				obj_info->object_type = g_strdup (name);
@@ -716,7 +735,7 @@ get_object_info (Accessible *accessible)
 			SPI_freeString (name);
 		}
 		else {
-			obj_info->prefix = g_strdup ("ukn");
+			obj_info->prefix = g_strdup ("unk");
 			obj_info->instance_index = unknown;
 			unknown++;
 			obj_info->object_type = g_strdup ("unknown");
@@ -724,7 +743,7 @@ get_object_info (Accessible *accessible)
 	}
 	else {
 		char *name;
-		obj_info->prefix = g_strdup ("ukn");
+		obj_info->prefix = g_strdup ("unk");
 		obj_info->instance_index = unknown;
 		unknown++;
 		name = Accessible_getRoleName (accessible);
@@ -743,7 +762,7 @@ get_object_info (Accessible *accessible)
 static char*
 add_appmap_data (Accessible *accessible, char *parent_name, long child_index,
 		 GHashTable *current_context, gboolean overloaded_window_name,
-		 gboolean remap_context)
+		 gboolean remap_context, gchar *curr_context_name)
 {
 	gchar *name = NULL;
 	gchar *label = NULL;
@@ -799,8 +818,17 @@ add_appmap_data (Accessible *accessible, char *parent_name, long child_index,
 		} else
 			label = g_strdup (label_by);
 	} else {
-		if (accessible_name)
+		if (accessible_name && \
+		    g_ascii_strcasecmp (accessible_name, "") != 0 &&
+		    g_ascii_strcasecmp (accessible_name, "...") != 0 )
+			// Ugly hack specific to VMware, as the team toggle
+			// button which has label as ...
 			label = g_strdup (accessible_name);
+		else {
+			if (accessible_desc && \
+			    g_ascii_strcasecmp (accessible_desc, "") != 0)
+				label = g_strdup (accessible_desc);
+		}
 	}
 
 	key_binding = get_keybinding (accessible);
@@ -933,8 +961,11 @@ add_appmap_data (Accessible *accessible, char *parent_name, long child_index,
 			}
 		}
 		if (ldtp_debug && name && tmp_parent_name)
-			g_print ("Stripped name, prefix, index: %s - Parent: %s\n", name, tmp_parent_name);
+			g_print ("Stripped name, prefix, index: %s - Parent: %s %d\n",
+				 name, tmp_parent_name, __LINE__);
 		g_free (stripped_data);
+	} else if (curr_context_name) {
+		name = g_strdup (curr_context_name);
 	} else if (overloaded_window_name) {
 		name = g_strdup (accessible_name);
 	} else {
@@ -971,9 +1002,19 @@ add_appmap_data (Accessible *accessible, char *parent_name, long child_index,
 	}
 
 	if (ldtp_debug && name && tmp_parent_name)
-		g_print ("Stripped name, prefix, index: %s - Parent: %s\n", name, tmp_parent_name);
+		g_print ("Stripped name, prefix, index: %s - Parent: %s %d\n",
+			 name, tmp_parent_name, __LINE__);
 	g_free (label);
 	label = g_strdup (accessible_name);
+	if (accessible_name &&						\
+	    g_ascii_strcasecmp (accessible_name, "...") != 0 )
+		// Ugly hack specific to VMware, as the team toggle
+		// button which has label as ...
+		label = g_strdup (accessible_name);
+	else {
+		if (accessible_desc)
+			label = g_strdup (accessible_desc);
+	}
 	if (label) {
 		gchar *key_binding = NULL;
        
@@ -1017,6 +1058,8 @@ add_appmap_data (Accessible *accessible, char *parent_name, long child_index,
 							 destroy_string, destroy_string);
 		if (hash_attributes) {
 			GHashTable *tmp_parent_table = NULL;
+			if (ldtp_debug && name && curr_context_name)
+				g_print ("Name: %s Current context name: %s\n", name, curr_context_name);
 			g_hash_table_insert (hash_attributes,
 					     g_strdup ("key"),
 					     g_strdup (name));
@@ -1037,13 +1080,17 @@ add_appmap_data (Accessible *accessible, char *parent_name, long child_index,
 							if (ldtp_debug)
 							g_print ("Children: %s\n", children);
 							//g_hash_table_remove (tmp_parent_table, "children");
-							g_hash_table_replace (tmp_parent_table, g_strdup ("children"), children);
+							g_hash_table_replace (tmp_parent_table,
+									      g_strdup ("children"),
+									      children);
 						} else {
 							if (ldtp_debug)
 								g_print ("Unable to create children");
 						}
 					} else {
-						g_hash_table_insert (tmp_parent_table, g_strdup ("children"), g_strdup (name));
+						g_hash_table_insert (tmp_parent_table,
+								     g_strdup ("children"),
+								     g_strdup (name));
 					}
 				} else {
 					if (ldtp_debug)
@@ -1056,7 +1103,8 @@ add_appmap_data (Accessible *accessible, char *parent_name, long child_index,
 			g_hash_table_insert (hash_attributes,
 					     g_strdup ("child_index"),
 					     g_strdup_printf ("%ld", child_index));
-			if (g_ascii_strcasecmp (label, "") != 0 && cur_obj_info->object_type &&
+			if (g_ascii_strcasecmp (label, "") != 0 &&
+			    cur_obj_info->object_type &&
 			    (g_ascii_strcasecmp (cur_obj_info->object_type,
 						 "combo_box") != 0 &&
 			     g_ascii_strcasecmp (cur_obj_info->object_type,
@@ -1066,7 +1114,9 @@ add_appmap_data (Accessible *accessible, char *parent_name, long child_index,
 						     g_strdup (label));
 			} else {
 				if (label_by && g_ascii_strcasecmp (label_by, "") != 0 ) {
-					g_hash_table_insert (hash_attributes, g_strdup ("label_by"), g_strdup (label_by));
+					g_hash_table_insert (hash_attributes,
+							     g_strdup ("label_by"),
+							     g_strdup (label_by));
 				}
 			}
 			if (g_ascii_strcasecmp (accessible_desc, "") != 0) {
@@ -1136,7 +1186,8 @@ add_appmap_data (Accessible *accessible, char *parent_name, long child_index,
 static void
 accessible_object_handle (Accessible *accessible, char *parent_name, 
 			  long child_index, GHashTable *current_context,
-			  gboolean overloaded_window_name, gboolean remap_context)
+			  gboolean overloaded_window_name, gboolean remap_context,
+			  gchar *curr_context_name)
 {
 	long i, num_child;
 	Accessible *child;
@@ -1147,6 +1198,8 @@ accessible_object_handle (Accessible *accessible, char *parent_name,
 	role = Accessible_getRoleName (accessible);
 	if (ldtp_debug && role && parent_name)
 		g_print ("Accessible: %s - %s - %ld\n", role, parent_name, child_index);
+	if (ldtp_debug && curr_context_name)
+		g_print ("Current context: %s\n", curr_context_name);
 	SPI_freeString (role);
 	num_child = Accessible_getChildCount (accessible);
 	if (child_index != -1) {
@@ -1154,14 +1207,16 @@ accessible_object_handle (Accessible *accessible, char *parent_name,
 			g_print ("Child Index: %ld\n", child_index);
 		temp_parent = add_appmap_data (accessible, parent_name, child_index,
 					       current_context, overloaded_window_name,
-					       remap_context);
+					       remap_context, curr_context_name);
 		if (overloaded_window_name)
 			overloaded_window_name = FALSE;
-	} else
+	} else {
 		temp_parent = parent_name;
+	}
 	if (temp_parent) {
 		if (ldtp_debug)
-			g_print ("Parent: %s - Child Index: %ld\n", temp_parent, child_index);
+			g_print ("Parent: %s - Child Index: %ld\n",
+				 temp_parent, child_index);
 		current_parent = temp_parent;
 		for (i = 0; i < num_child; i++) {
 			child = Accessible_getChildAtIndex (accessible, i);
@@ -1178,7 +1233,7 @@ accessible_object_handle (Accessible *accessible, char *parent_name,
 			accessible_object_handle (child, current_parent,
 						  i, current_context,
 						  overloaded_window_name,
-						  remap_context);
+						  remap_context, NULL);
 			Accessible_unref (child);
 		}
 		if (current_parent && parent_name && current_parent != parent_name)
@@ -1200,7 +1255,7 @@ get_window_name_in_appmap_format (char *window_name, AccessibleRole role)
 		type = g_strdup ("dlg");
 	else
 		type = g_strdup ("");
-	if (window_name && g_utf8_collate (window_name, "") != 0) {
+	if (window_name && g_ascii_strcasecmp (window_name, "") != 0) {
 		char *stripped_data = NULL;
 		if (g_utf8_strchr (window_name, -1, ' ') != NULL) {
 			stripped_data = escape_character (window_name, ' ');
@@ -1229,7 +1284,8 @@ get_window_name_in_appmap_format (char *window_name, AccessibleRole role)
 
 GHashTable *do_remap (Accessible *accessible_context, char *name, GHashTable *context,
 		      GHashTable *context_attributes, gboolean locale_flag,
-		      gboolean overloaded_window_name, gboolean remap_context)
+		      gboolean overloaded_window_name, gboolean remap_context,
+		      gchar *curr_context_name)
 {
 	GHashTable *current_context = NULL;
 
@@ -1241,21 +1297,26 @@ GHashTable *do_remap (Accessible *accessible_context, char *name, GHashTable *co
 	if (remap_context) {
 		accessible_object_handle (accessible_context, name,
 					  Accessible_getIndexInParent (accessible_context),
-					  context, overloaded_window_name, remap_context);
+					  context, overloaded_window_name, remap_context,
+					  curr_context_name);
 		return NULL;
 	}
-	current_context = g_hash_table_new_full (&g_str_hash, &g_str_equal, destroy_string, destroy_hashtable);
+	current_context = g_hash_table_new_full (&g_str_hash, &g_str_equal,
+						 destroy_string, destroy_hashtable);
 	if (context_attributes) {
 		if (ldtp_debug)
-			g_print ("Context Attributes NOT NULL - FLAG: %d\n", overloaded_window_name);
-		g_hash_table_insert (current_context, g_strdup (name), context_attributes);
+			g_print ("Context Attributes NOT NULL - FLAG: %d\n",
+				 overloaded_window_name);
+		g_hash_table_insert (current_context,
+				     g_strdup (name),
+				     context_attributes);
 		/*
 		 * NOTE: Passing -1 as parameter signifies that the first entry has
 		 * already been added, so should not do add_appmap_data for this call
 		 */
 		accessible_object_handle (accessible_context, name, -1,
 					  current_context, overloaded_window_name,
-					  remap_context);
+					  remap_context, curr_context_name);
 	} else {
 		char *role = Accessible_getRoleName (accessible_context);
 		long parent_index = Accessible_getIndexInParent (accessible_context);
@@ -1267,7 +1328,7 @@ GHashTable *do_remap (Accessible *accessible_context, char *name, GHashTable *co
 				 overloaded_window_name);
 		SPI_freeString (role);
 		if (parent_index == -1) {
-			// To over come the bug with Thunderbird !
+			// To over come the bug with Thunderbird or Gantt Chart Java application !
 			// For all frames, Accessible_getIndexInParent function returns -1
 			long i;
 			Accessible *child;
@@ -1299,7 +1360,8 @@ GHashTable *do_remap (Accessible *accessible_context, char *name, GHashTable *co
 		}
 		accessible_object_handle (accessible_context, name,
 					  parent_index, current_context,
-					  overloaded_window_name, remap_context);
+					  overloaded_window_name, remap_context,
+					  curr_context_name);
 	}
 	return current_context;
 }
