@@ -29,7 +29,8 @@
 gchar *strip_delim (const gchar *text, gchar ch);
 GHashTable *do_remap (Accessible *accessible, char *name, GHashTable *context,
 		      GHashTable *context_attributes, gboolean locale_flag,
-		      gboolean overloadedWindowName, gboolean remap_context); 
+		      gboolean overloadedWindowName, gboolean remap_context,
+		      gchar *curr_context_name); 
 char *get_window_name_in_appmap_format (char *window_name, AccessibleRole role);
 
 #endif
