@@ -3,9 +3,10 @@
  * Linux Desktop Testing Project http://ldtp.freedesktop.org
  *
  * Author:
- *    Nagappan A <nagappan@gmail.com>
+ *    Nagappan Alagappan <nagappan@gmail.com>
  *
  * Copyright 2004 - 2006 Novell, Inc.
+ * Copyright 2007 - 2008 Nagappan Alagappan
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -58,7 +59,7 @@ sort_column_index (Accessible *object, int col, FILE *log_fp)
 			g_sprintf (msg, "Column %d doesnot exist in table", col);
 			log_msg (LDTP_LOG_CAUSE, msg, log_fp);
 			Accessible_unref (table);
-			error =  (LDTP_ERROR_INVALID_COLUMN_INDEX_TO_SORT);
+			error = LDTP_ERROR_INVALID_COLUMN_INDEX_TO_SORT;
 			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 			return error;
 	}
@@ -82,11 +83,11 @@ sort_column_index (Accessible *object, int col, FILE *log_fp)
 	SPI_freeString (name);
 
 	if (flag == FALSE) { 
-		error =  (LDTP_ERROR_UNABLE_TO_SORT);
+		error = LDTP_ERROR_UNABLE_TO_SORT;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
-	return  (LDTP_ERROR_SUCCESS);
+	return LDTP_ERROR_SUCCESS;
 }  
 	
 /*
@@ -126,7 +127,7 @@ sort_column (Accessible *object, char *column_header_text, FILE *log_fp)
 		if (table)
 			Accessible_unref (table);
 		Accessible_unref (header);
-		error =  (LDTP_ERROR_COLUMN_DOES_NOT_EXIST);
+		error = LDTP_ERROR_COLUMN_DOES_NOT_EXIST;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
@@ -150,13 +151,130 @@ sort_column (Accessible *object, char *column_header_text, FILE *log_fp)
 	SPI_freeString (name);
 
 	if (flag == FALSE) { 
-		error =  (LDTP_ERROR_UNABLE_TO_SORT);
+		error = LDTP_ERROR_UNABLE_TO_SORT;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
-	return  (LDTP_ERROR_SUCCESS);
+	return LDTP_ERROR_SUCCESS;
 }  
 
+static LDTPErrorCode
+right_click (Accessible *object, GSList *l, FILE *log_fp)
+{
+	long i, j;
+	LDTPErrorCode error;
+	gboolean flag = FALSE;
+	long n_rows, n_cols;
+	Accessible *cell;
+	AccessibleTable *table;
+	char *name = NULL;
+	char *row_text = NULL;
+	long n_matches = 0, matches = 0;
+	long x, y, height, width;
+
+	if (l && l->data)
+		row_text = l->data;
+	else {
+		error = LDTP_ERROR_RIGHT_CLICK_FAILED;
+		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
+		return error;
+	}
+	l = l->next;
+	if (l && l->data)
+		n_matches = atol (l->data);
+    
+	table = Accessible_getTable (object);
+	n_rows = AccessibleTable_getNRows (table);
+	n_cols = AccessibleTable_getNColumns (table);
+	g_print ("Number of rows: %ld\tColumn: %ld\n", n_rows, n_cols);
+
+	for (i = 0; i < n_rows; i++) {
+		for (j = 0; j < n_cols; j++) {
+			long child_count;
+			cell = AccessibleTable_getAccessibleAt (table, i, j);
+			child_count = Accessible_getChildCount (cell);
+			g_print ("i = %ld, j = %ld, child_count: %ld\n", i, j, child_count);
+			if (child_count > 0) {
+				long i;
+				Accessible *child;
+				for (i = 0; i < child_count; i++) {
+					child = Accessible_getChildAtIndex (cell, i);
+					name = Accessible_getName (child);
+					if (ldtp_debug && name && row_text)
+						g_print ("Name: *%s* - Row: *%s*\n", name, row_text);
+					if (g_utf8_collate (name, row_text) == 0) {
+						g_print ("Matches: %ld - n_matches: %ld\n", matches, n_matches);
+						if (n_matches != matches) {
+							matches++;
+							Accessible_unref (child);
+							g_print ("Continue\n");
+							continue;
+						} 
+
+						if (Accessible_isComponent (cell)) {
+							AccessibleComponent *accessible_component;
+							accessible_component = Accessible_getComponent (cell);
+							AccessibleComponent_getExtents (accessible_component,
+											&x, &y, &height, &width,
+											SPI_COORD_TYPE_SCREEN);
+							AccessibleComponent_grabFocus (accessible_component);
+							flag = TRUE;
+							Accessible_unref (accessible_component);
+						}
+						SPI_freeString (name);
+						Accessible_unref (child);
+						Accessible_unref (cell);
+						break;
+					} // if
+					SPI_freeString (name);
+					Accessible_unref (child);
+				} // for
+			} else {
+				name = Accessible_getName (cell);
+				g_print ("name: %s row_text: %s\n", name, row_text);
+				if (g_utf8_collate (name, row_text) == 0) {
+					if (Accessible_isComponent (cell)) {
+						AccessibleComponent *accessible_component;
+						accessible_component = Accessible_getComponent (cell);
+						/*
+						AccessibleComponent_getExtents (accessible_component,
+										&x, &y, &height, &width,
+										SPI_COORD_TYPE_SCREEN);
+						*/
+						AccessibleComponent_getPosition (accessible_component,
+										&x, &y,
+										SPI_COORD_TYPE_SCREEN);
+						AccessibleComponent_getSize (accessible_component,
+										 &height, &width);
+						AccessibleComponent_grabFocus (accessible_component);
+						flag = TRUE;
+						Accessible_unref (accessible_component);
+					}
+					SPI_freeString (name);
+					Accessible_unref (cell);
+					break;
+				}
+			}
+			Accessible_unref (cell);
+		} // for j
+		if (flag == TRUE)
+			break;
+	} // for i
+	Accessible_unref (table);
+	if (flag) {
+		g_print ("x=%ld, y=%ld, width=%ld, height=%ld\n", x, y, width, height);
+		/*
+		if (SPI_generateMouseEvent (x + width / 2, y + height / 2, "b3c"))
+			return LDTP_ERROR_SUCCESS;
+		*/
+		if (SPI_generateMouseEvent (x, y + 2, "b3c"))
+			return LDTP_ERROR_SUCCESS;
+	}
+	error = LDTP_ERROR_RIGHT_CLICK_FAILED;
+	log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
+	return error;
+}
+
 static Accessible*
 get_row (Accessible *object, char *row_text, long *row, long *column, FILE *log_fp)
 {
@@ -289,17 +407,17 @@ get_cell_value (LDTPClientContext* cctxt)
 	table = Accessible_getTable (object);
 	n_rows = AccessibleTable_getNRows (table);
 	n_cols = AccessibleTable_getNColumns (table);
-	g_print ("Rows: %ld -- Columns: %ld", n_rows, n_cols);
+	g_print ("Rows: %ld -- Columns: %ld\n", n_rows, n_cols);
 
 	if (n_rows < row) {
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_ACTUAL_ROW_COUNT_LESS_THAN_GIVEN_ROW_COUNT);
+		error = LDTP_ERROR_ACTUAL_ROW_COUNT_LESS_THAN_GIVEN_ROW_COUNT;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 		return error;
 	}
 	if (n_cols < column) {
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_ACTUAL_COLUMN_COUNT_LESS_THAN_GIVEN_COLUMN_COUNT);
+		error = LDTP_ERROR_ACTUAL_COLUMN_COUNT_LESS_THAN_GIVEN_COLUMN_COUNT;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 		return error;
 	}
@@ -307,12 +425,11 @@ get_cell_value (LDTPClientContext* cctxt)
 	parent_cell = AccessibleTable_getAccessibleAt (table, row, column);
 	child_count = Accessible_getChildCount (parent_cell);
 	child_object = NULL;
-  
+
 	if (child_count == 0) {
 		flag = TRUE;
 		child_object = parent_cell;
-	}
-	else {
+	} else {
 		for (i = 0; i < child_count && flag == 0; i++) {
 			child_level2 = Accessible_getChildAtIndex (parent_cell, i);
 			child_count_level2 = Accessible_getChildCount (child_level2);
@@ -326,8 +443,7 @@ get_cell_value (LDTPClientContext* cctxt)
 					}
 					Accessible_unref (child_level3);
 				}
-			}
-			else {
+			} else {
 				if (Accessible_isText (child_level2)) {
 					flag = TRUE;
 					child_object = child_level2;
@@ -342,7 +458,7 @@ get_cell_value (LDTPClientContext* cctxt)
 	if (flag == TRUE) {
 		if (Accessible_isComponent (child_object)) {
 			child_type = get_object_type (child_object);
-			g_print ("Child type is %d ", child_type);
+			g_print ("Child type is %d\n", child_type);
 			if (child_type == SPI_ROLE_TEXT || child_type == SPI_ROLE_TABLE_CELL) {
 				if (Accessible_isText (child_object)) {
 					g_print ("Table cell is a text object\n");
@@ -356,25 +472,26 @@ get_cell_value (LDTPClientContext* cctxt)
 					Accessible_unref (child_object);
 					cctxt->gui_handle->handle = object;
 					return error;
-				}
-			}
+				} else
+					g_print ("child_object is not Text\n");
+			} else
+				g_print ("Unsupported child object type\n");
 		} //if
 		Accessible_unref (table);
 		if (child_object)
 			Accessible_unref (child_object);
 		cctxt->gui_handle->handle = object;
-	}
-	else {
+	} else {
 		log_msg (LDTP_LOG_CAUSE, "Table cell has no child of type TEXT", cctxt->log_fp);
 		Accessible_unref (table);
 		if (child_object)
 			Accessible_unref (child_object);
 		cctxt->gui_handle->handle = object;
-		error =  (LDTP_ERROR_NO_CHILD_TEXT_TYPE_UNDER_TABLE);
+		error = LDTP_ERROR_NO_CHILD_TEXT_TYPE_UNDER_TABLE;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 		return error;
 	}
-	error =  (LDTP_ERROR_GET_TABLE_CELL_FAILED);
+	error = LDTP_ERROR_GET_TABLE_CELL_FAILED;
 	log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 	return error;
 }
@@ -394,7 +511,7 @@ select_row (LDTPClientContext *cctxt, Accessible *object, char *row_text, FILE *
 
 	if (!object) {
 		g_print ("%s - %d\n", __FILE__, __LINE__);
-		error =  (LDTP_ERROR_UNABLE_TO_SELECT_ROW);
+		error = LDTP_ERROR_UNABLE_TO_SELECT_ROW;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
@@ -406,7 +523,7 @@ select_row (LDTPClientContext *cctxt, Accessible *object, char *row_text, FILE *
 	cell = get_row (object, row_text, &row, &column, log_fp);
 	if (cell) {
 		if (Accessible_isComponent (cell)) {
-			SPIBoolean flag;
+			SPIBoolean flag = FALSE;
 			AccessibleComponent *accessible_component;
 			start = cur = time (NULL);
 			while (difftime (cur, start) < time_wait) {
@@ -415,28 +532,27 @@ select_row (LDTPClientContext *cctxt, Accessible *object, char *row_text, FILE *
 					flag = AccessibleComponent_grabFocus (accessible_component);
 					Accessible_unref (accessible_component);
 				}
-				if (flag) {
-					if (Accessible_isText (cell)) {
-						AccessibleText *text = Accessible_getText (cell);
-						if (text) {
-							char *name = AccessibleText_getText (text, 0, LONG_MAX);
-							if (name) {
-								g_print ("Cell content: %s\n", name);
-								SPI_freeString (name);
-							}
-							Accessible_unref (text);
+				if (!flag) {
+					ldtp_nsleep (0, 30000);
+					cur = time (NULL);
+				}
+				if (Accessible_isText (cell)) {
+					AccessibleText *text = Accessible_getText (cell);
+					if (text) {
+						char *name = AccessibleText_getText (text, 0, LONG_MAX);
+						if (name) {
+							g_print ("Cell content: %s\n", name);
+							SPI_freeString (name);
 						}
+						Accessible_unref (text);
 					}
 					break;
 				}
-				ldtp_nsleep (0, 30000);
-
-				cur = time (NULL);
 			}
 			Accessible_unref (cell);
 			if (flag) {
 				g_print ("Grabed focus\n");
-				return  (LDTP_ERROR_SUCCESS);
+				return LDTP_ERROR_SUCCESS;
 			} else {
 				g_print ("Unable to grab focus\n");
 				error = LDTP_ERROR_UNABLE_TO_GRAB_FOCUS;
@@ -446,12 +562,12 @@ select_row (LDTPClientContext *cctxt, Accessible *object, char *row_text, FILE *
 		}
 		else {
 			Accessible_unref (cell);
-			error =  (LDTP_ERROR_OBJ_NOT_COMPONENT_TYPE);
+			error = LDTP_ERROR_OBJ_NOT_COMPONENT_TYPE;
 			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 			return error;
 		}
 	}
-	error =  (LDTP_ERROR_UNABLE_TO_SELECT_ROW);
+	error = LDTP_ERROR_UNABLE_TO_SELECT_ROW;
 	log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 	return error;
 }
@@ -471,7 +587,7 @@ double_click (Accessible *object, FILE *log_fp)
 		component = Accessible_getComponent (object);
 	else {
 		Accessible_unref (object);
-		error =  (LDTP_ERROR_OBJ_NOT_COMPONENT_TYPE);
+		error = LDTP_ERROR_OBJ_NOT_COMPONENT_TYPE;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
@@ -485,15 +601,15 @@ double_click (Accessible *object, FILE *log_fp)
 	flag = AccessibleComponent_grabFocus (component);
 	Accessible_unref (component);
 	if (!flag) {
-		error =  (LDTP_ERROR_UNABLE_TO_GRAB_FOCUS);
+		error = LDTP_ERROR_UNABLE_TO_GRAB_FOCUS;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
 	flag = SPI_generateMouseEvent (x, y, "b1d");
 	if (flag)
-		return  (LDTP_ERROR_SUCCESS);
+		return LDTP_ERROR_SUCCESS;
 	else {
-		error =  (LDTP_ERROR_DOUBLE_CLICK_FAILED);
+		error = LDTP_ERROR_DOUBLE_CLICK_FAILED;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
@@ -520,7 +636,7 @@ double_click_row (Accessible *object, char *row_text, FILE *log_fp)
 			component = Accessible_getComponent (cell);
 		else {
 			Accessible_unref (cell);
-			error =  (LDTP_ERROR_OBJ_NOT_COMPONENT_TYPE);
+			error = LDTP_ERROR_OBJ_NOT_COMPONENT_TYPE;
 			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 			return error;
 		}
@@ -535,20 +651,20 @@ double_click_row (Accessible *object, char *row_text, FILE *log_fp)
 		Accessible_unref (cell);
 		Accessible_unref (component);
 		if (!flag) {
-			error =  (LDTP_ERROR_UNABLE_TO_GRAB_FOCUS);
+			error = LDTP_ERROR_UNABLE_TO_GRAB_FOCUS;
 			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 			return error;
 		}
 		flag = SPI_generateMouseEvent (x, y, "b1d");
 		if (flag)
-			return  (LDTP_ERROR_SUCCESS);
+			return LDTP_ERROR_SUCCESS;
 		else {
-			error =  (LDTP_ERROR_DOUBLE_CLICK_FAILED);
+			error = LDTP_ERROR_DOUBLE_CLICK_FAILED;
 			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 			return error;
 		}
 	}
-	error =  (LDTP_ERROR_UNABLE_TO_GET_CELL_HANDLE_FAILED);
+	error = LDTP_ERROR_UNABLE_TO_GET_CELL_HANDLE_FAILED;
 	log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 	return error;
 }
@@ -574,7 +690,7 @@ select_row_index (Accessible *object, long row, FILE *log_fp)
 		g_sprintf (msg, "Row %ld doesnot exist in table", row);
 		log_msg (LDTP_LOG_CAUSE, msg, log_fp);
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_ROW_DOES_NOT_EXIST);
+		error = LDTP_ERROR_ROW_DOES_NOT_EXIST;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
@@ -592,8 +708,8 @@ select_row_index (Accessible *object, long row, FILE *log_fp)
 		Accessible_unref (cell);
 		Accessible_unref (table);
 		if (flag)
-			return  (LDTP_ERROR_SUCCESS);
-		error =  (LDTP_ERROR_UNABLE_TO_GRAB_FOCUS);
+			return LDTP_ERROR_SUCCESS;
+		error = LDTP_ERROR_UNABLE_TO_GRAB_FOCUS;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	} else 	{
@@ -602,7 +718,7 @@ select_row_index (Accessible *object, long row, FILE *log_fp)
 		g_print ("Table not Accessible component\n");
 		Accessible_unref (cell);
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_OBJ_NOT_COMPONENT_TYPE);
+		error = LDTP_ERROR_OBJ_NOT_COMPONENT_TYPE;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
@@ -633,16 +749,16 @@ select_last_row (Accessible *object, FILE *log_fp)
 		Accessible_unref (cell);
 		Accessible_unref (table);
 		if (flag == TRUE)
-			return  (LDTP_ERROR_SUCCESS);
+			return LDTP_ERROR_SUCCESS;
 		else {
-			error =  (LDTP_ERROR_UNABLE_TO_GRAB_FOCUS);
+			error = LDTP_ERROR_UNABLE_TO_GRAB_FOCUS;
 			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 			return error;
 		}
 	} else 	{
 		Accessible_unref (cell);
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_OBJ_NOT_COMPONENT_TYPE);
+		error = LDTP_ERROR_OBJ_NOT_COMPONENT_TYPE;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
@@ -665,10 +781,10 @@ does_row_exist (Accessible *object, char *row_text, FILE *log_fp)
 		if (n_rows != 0) {
 			log_msg (LDTP_LOG_INFO, "Rows available in Table cell", log_fp);
 			Accessible_unref (table);
-			return  (LDTP_ERROR_SUCCESS);
+			return LDTP_ERROR_SUCCESS;
 		}
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_ROW_DOES_NOT_EXIST);
+		error = LDTP_ERROR_ROW_DOES_NOT_EXIST;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	} else {
@@ -715,9 +831,9 @@ does_row_exist (Accessible *object, char *row_text, FILE *log_fp)
 		Accessible_unref (table);
 	} // else
 	if (flag == TRUE)
-		return  (LDTP_ERROR_SUCCESS);
+		return LDTP_ERROR_SUCCESS;
 	else {
-		error =  (LDTP_ERROR_ROW_DOES_NOT_EXIST);
+		error = LDTP_ERROR_ROW_DOES_NOT_EXIST;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
@@ -729,15 +845,19 @@ verify_cell_value (LDTPClientContext* cctxt)
 	char *text;
 	int child_type;
 	long row = 0, column = 0;
-	long n_rows, n_cols;
+	long i, n_rows, n_cols;
 	long n_actions, j;
 	long offset, child_count;
 	char *name = NULL;
 	GSList *l;
 	LDTPErrorCode error;
-	AccessibleAction *action;
+	gboolean flag = FALSE;
 	AccessibleTable *table;
+	Accessible *parent_cell;
+	AccessibleAction *action;
 	Accessible *child_object;
+	long child_count_level2;
+	Accessible *child_level2, *child_level3;
 	Accessible *object = cctxt->gui_handle->handle;
   
 	l = cctxt->req->arg_list;
@@ -755,28 +875,49 @@ verify_cell_value (LDTPClientContext* cctxt)
  
 	if (n_rows < row) {
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_ACTUAL_ROW_COUNT_LESS_THAN_GIVEN_ROW_COUNT);
+		error = LDTP_ERROR_ACTUAL_ROW_COUNT_LESS_THAN_GIVEN_ROW_COUNT;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 		return error;
 	}
 	if (n_cols < column) {
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_ACTUAL_COLUMN_COUNT_LESS_THAN_GIVEN_COLUMN_COUNT);
+		error = LDTP_ERROR_ACTUAL_COLUMN_COUNT_LESS_THAN_GIVEN_COLUMN_COUNT;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 		return error;
 	}  
 	g_print ("Table - number of rows:%ld\tColumn: %ld\n", n_rows, n_cols);
-	child_object = AccessibleTable_getAccessibleAt (table, row, column);
+	parent_cell = AccessibleTable_getAccessibleAt (table, row, column);
+	child_count = Accessible_getChildCount (parent_cell);
+	child_object = NULL;
 
-	do {
-		child_count = Accessible_getChildCount (child_object);
-		if (child_count > 0) {
-			Accessible *tmp_child;
-			tmp_child = child_object;
-			child_object = Accessible_getChildAtIndex (child_object, 0);
-			Accessible_unref (tmp_child);
+	if (child_count == 0) {
+		flag = TRUE;
+		child_object = parent_cell;
+	} else {
+		for (i = 0; i < child_count && flag == 0; i++) {
+			child_level2 = Accessible_getChildAtIndex (parent_cell, i);
+			child_count_level2 = Accessible_getChildCount (child_level2);
+			if (child_count_level2 > 0) {
+				for (j = 0; j < child_count_level2 && flag == FALSE; j++) {
+					child_level3 = Accessible_getChildAtIndex (child_level2, j);
+					if (Accessible_isText (child_level3)) {
+						flag = TRUE;
+						child_object = child_level3;
+						break;
+					}
+					Accessible_unref (child_level3);
+				}
+			} else {
+				if (Accessible_isText (child_level2)) {
+					flag = TRUE;
+					child_object = child_level2;
+				}
+			}
+			if (flag == FALSE)
+				Accessible_unref (child_level2);
 		}
-	} while (child_count > 0);
+		Accessible_unref (parent_cell);
+	}
 
 	cctxt->gui_handle->handle = child_object;
 	if (Accessible_isComponent (child_object)) {
@@ -785,12 +926,11 @@ verify_cell_value (LDTPClientContext* cctxt)
 		if (child_type == SPI_ROLE_TABLE_CELL) {
 			action = Accessible_getAction (child_object);
 			n_actions = AccessibleAction_getNActions (action);
-			if (n_actions) {
+			g_print ("Actions: %ld\n", n_actions);
+			if (n_actions > 0) {
 				for (j = 0; j < n_actions; j++) {
-					/*
-					  FIXME: Use Accessible_getRole
-					*/
 					name = AccessibleAction_getName (action, j);
+					g_print ("Action name: %s\n", name);
 					/* Handled at present only Toggle & Text type of cell */
 					if (g_ascii_strcasecmp (name, "toggle") == 0) {
 						GSList *l = g_slist_last (cctxt->req->arg_list);
@@ -805,7 +945,8 @@ verify_cell_value (LDTPClientContext* cctxt)
 						cctxt->gui_handle->handle = object;
 						return error;
 					}
-					if (g_ascii_strcasecmp (name, "edit") == 0) {
+					if (g_ascii_strcasecmp (name, "edit") == 0 || \
+					    g_ascii_strcasecmp (name, "activate") == 0) {
 						GSList *l = g_slist_last (cctxt->req->arg_list);
 						cctxt->req->arg_list = g_slist_prepend (cctxt->req->arg_list, l->data);
 						g_print ("Table Table cell is text box\n");
@@ -821,8 +962,13 @@ verify_cell_value (LDTPClientContext* cctxt)
 					SPI_freeString (name);
 				} //for 
 				Accessible_unref (action);
-			} //if
-			else {	
+			} else {
+				char *tmp_name = Accessible_getName (child_object);
+				char *tmp_role = Accessible_getRoleName (child_object);
+				g_print ("Name: %s - Role: %s\n", tmp_name, tmp_role);
+				SPI_freeString (tmp_name);
+				SPI_freeString (tmp_role);
+				g_print ("ELSE %d\n", Accessible_isText (child_object));
 				if (Accessible_isText (child_object)) {	
 					AccessibleText *text_object;
 
@@ -847,8 +993,7 @@ verify_cell_value (LDTPClientContext* cctxt)
 				SPI_freeString (name);
 				Accessible_unref (action);
 			} // else
-		} // if (child_type == SPI_ROLE_TABLE_CELL)
-		else if (child_type == SPI_ROLE_TEXT) {
+		} else if (child_type == SPI_ROLE_TEXT) {
 			GSList *l = g_slist_last (cctxt->req->arg_list);
 			cctxt->req->arg_list = g_slist_prepend (cctxt->req->arg_list, l->data);
 			g_print ("Table cell is text box \n");
@@ -858,8 +1003,7 @@ verify_cell_value (LDTPClientContext* cctxt)
 			Accessible_unref (child_object);
 			cctxt->gui_handle->handle = object;
 			return error;  
-		}
-		else if (child_type == SPI_ROLE_TOGGLE_BUTTON) {
+		} else if (child_type == SPI_ROLE_TOGGLE_BUTTON) {
 			GSList *l = g_slist_last (cctxt->req->arg_list);
 			cctxt->req->arg_list = g_slist_prepend (cctxt->req->arg_list, l->data);
 			g_print ("Table cell is of type Toggle\n");
@@ -874,7 +1018,7 @@ verify_cell_value (LDTPClientContext* cctxt)
 	Accessible_unref (table);
 	Accessible_unref (child_object);
 	cctxt->gui_handle->handle = object;
-	error =  (LDTP_ERROR_VERIFY_TABLE_CELL_FAILED);
+	error = LDTP_ERROR_VERIFY_TABLE_CELL_FAILED;
 	log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 	return error;
 }  
@@ -908,13 +1052,13 @@ set_cell_value (LDTPClientContext* cctxt)
  
 	if (n_rows < row) {
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_ACTUAL_ROW_COUNT_LESS_THAN_GIVEN_ROW_COUNT);
+		error = LDTP_ERROR_ACTUAL_ROW_COUNT_LESS_THAN_GIVEN_ROW_COUNT;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 		return error;
 	}
 	if (n_cols < column) {
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_ACTUAL_COLUMN_COUNT_LESS_THAN_GIVEN_COLUMN_COUNT);
+		error = LDTP_ERROR_ACTUAL_COLUMN_COUNT_LESS_THAN_GIVEN_COLUMN_COUNT;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 		return error;
 	}
@@ -930,11 +1074,8 @@ set_cell_value (LDTPClientContext* cctxt)
 			name = AccessibleAction_getName (action, j);
 			/* Handled at present only Toggle & Text type of cell */
 			if (g_ascii_strcasecmp (name, "toggle") == 0) {
-				GSList *l = g_slist_last (cctxt->req->arg_list);
-				cctxt->req->arg_list = g_slist_prepend (cctxt->req->arg_list, l->data);
 				g_print ("Table cell is of type Toggle\n");
 				error = toggle_button_main (cctxt, LDTP_CMD_CLICK);
-				cctxt->req->arg_list = g_slist_remove (cctxt->req->arg_list, l->data);
 				SPI_freeString (name);
 				Accessible_unref (action);
 				Accessible_unref (table);
@@ -943,11 +1084,13 @@ set_cell_value (LDTPClientContext* cctxt)
 				return error;
 			}
 			if (g_ascii_strcasecmp (name, "edit") == 0) {
-				GSList *l = g_slist_last (cctxt->req->arg_list);
-				cctxt->req->arg_list = g_slist_prepend (cctxt->req->arg_list, l->data);
+				GSList *tmp_list = g_slist_last (cctxt->req->arg_list);
+				cctxt->req->arg_list = g_slist_prepend (cctxt->req->arg_list,
+									tmp_list->data);
 				g_print ("Table cell is text box %s - %d\n", __FILE__, __LINE__);
 				error = text_main (cctxt, LDTP_CMD_SETTEXTVALUE);
-				cctxt->req->arg_list = g_slist_remove (cctxt->req->arg_list, l->data);
+				cctxt->req->arg_list = g_slist_remove (cctxt->req->arg_list,
+								       tmp_list->data);
 				SPI_freeString (name);
 				Accessible_unref (action);
 				Accessible_unref (table);
@@ -963,7 +1106,7 @@ set_cell_value (LDTPClientContext* cctxt)
 	if (cell)
 		Accessible_unref (cell);
 	cctxt->gui_handle->handle = object;
-	error =  (LDTP_ERROR_SET_TABLE_CELL_FAILED);
+	error = LDTP_ERROR_SET_TABLE_CELL_FAILED;
 	log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 	return error;
 }  
@@ -990,7 +1133,7 @@ check_row (Accessible *object, int row, int col, FILE *log_fp)
 		g_sprintf (msg, "Row %d doesnot exist in table", row);
 		log_msg (LDTP_LOG_CAUSE, msg, log_fp);
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_ROW_DOES_NOT_EXIST);
+		error = LDTP_ERROR_ROW_DOES_NOT_EXIST;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
@@ -998,7 +1141,7 @@ check_row (Accessible *object, int row, int col, FILE *log_fp)
 		g_sprintf (msg, "Column %d doesnot exist in table", col);
 		log_msg (LDTP_LOG_CAUSE, msg, log_fp);
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_COLUMN_DOES_NOT_EXIST);
+		error = LDTP_ERROR_COLUMN_DOES_NOT_EXIST;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
@@ -1020,11 +1163,11 @@ check_row (Accessible *object, int row, int col, FILE *log_fp)
 				SPI_freeString (name);
 				AccessibleStateSet_unref (state);
 				log_msg (LDTP_LOG_WARNING, "Row is already checked", log_fp);
-				return  (LDTP_ERROR_SUCCESS);
+				return LDTP_ERROR_SUCCESS;
 			}
 			AccessibleStateSet_unref (state);
 			if (AccessibleAction_doAction (action, j) == FALSE) {
-				error =  (LDTP_ERROR_CHECK_ACTION_FAILED);
+				error = LDTP_ERROR_CHECK_ACTION_FAILED;
 				log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 				return error;
 			}
@@ -1034,7 +1177,7 @@ check_row (Accessible *object, int row, int col, FILE *log_fp)
 		Accessible_unref (cell);
 	Accessible_unref (action);
 	SPI_freeString (name);
-	return  (LDTP_ERROR_SUCCESS);
+	return LDTP_ERROR_SUCCESS;
 }
 
 static LDTPErrorCode
@@ -1059,7 +1202,7 @@ uncheck_row (Accessible *object, int row, int col, FILE *log_fp)
 		g_sprintf (msg, "Row %d doesnot exist in table", row);
 		log_msg (LDTP_LOG_CAUSE, msg, log_fp);
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_ROW_DOES_NOT_EXIST);
+		error = LDTP_ERROR_ROW_DOES_NOT_EXIST;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
@@ -1068,7 +1211,7 @@ uncheck_row (Accessible *object, int row, int col, FILE *log_fp)
 		g_sprintf (msg, "Column %d doesnot exist in table", col);
 		log_msg (LDTP_LOG_CAUSE, msg, log_fp);
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_COLUMN_DOES_NOT_EXIST);
+		error = LDTP_ERROR_COLUMN_DOES_NOT_EXIST;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
@@ -1090,11 +1233,11 @@ uncheck_row (Accessible *object, int row, int col, FILE *log_fp)
 				SPI_freeString (name);
 				AccessibleStateSet_unref (state);
 				log_msg (LDTP_LOG_WARNING, "Row is already unchecked", log_fp);
-				return  (LDTP_ERROR_SUCCESS);
+				return LDTP_ERROR_SUCCESS;
 			}
 			AccessibleStateSet_unref (state);
 			if (AccessibleAction_doAction (action, j) == FALSE) {
-				error =  (LDTP_ERROR_UNCHECK_ACTION_FAILED);
+				error = LDTP_ERROR_UNCHECK_ACTION_FAILED;
 				log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 				return error;
 			}
@@ -1104,7 +1247,7 @@ uncheck_row (Accessible *object, int row, int col, FILE *log_fp)
 		Accessible_unref (cell);
 	Accessible_unref (action);
 	SPI_freeString (name);
-	return  (LDTP_ERROR_SUCCESS);
+	return LDTP_ERROR_SUCCESS;
 }
 
 static LDTPErrorCode
@@ -1127,7 +1270,7 @@ get_row_count (Accessible *object, GSList **l)
 	*/
 	Accessible_unref (table);
 	*l = g_slist_prepend (*l, g_strdup_printf ("%ld", n_rows));
-	return  (LDTP_ERROR_SUCCESS);
+	return LDTP_ERROR_SUCCESS;
 }
 
 static LDTPErrorCode
@@ -1170,7 +1313,7 @@ select_row_partial_match (Accessible *object, char *row_text, FILE *log_fp)
 							if (flag == FALSE) {
 								Accessible_unref (child);
 								Accessible_unref (cell);
-								error =  (LDTP_ERROR_UNABLE_TO_GRAB_FOCUS);
+								error = LDTP_ERROR_UNABLE_TO_GRAB_FOCUS;
 								log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 								return error;
 							}
@@ -1199,7 +1342,7 @@ select_row_partial_match (Accessible *object, char *row_text, FILE *log_fp)
 						Accessible_unref (accessible_component);
 						if (flag == FALSE) {
 							Accessible_unref (cell);
-							error =  (LDTP_ERROR_UNABLE_TO_GRAB_FOCUS);
+							error = LDTP_ERROR_UNABLE_TO_GRAB_FOCUS;
 							log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 							SPI_freeString (name);
 							return error;
@@ -1220,18 +1363,19 @@ select_row_partial_match (Accessible *object, char *row_text, FILE *log_fp)
 	if (flag == FALSE) {
 		g_sprintf (msg, "Unable to find %s in table", row_text);
 		log_msg (LDTP_LOG_CAUSE, msg, log_fp);
-		error =  (LDTP_ERROR_UNABLE_TO_SELECT_ROW);
+		error = LDTP_ERROR_UNABLE_TO_SELECT_ROW;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
-	return  (LDTP_ERROR_SUCCESS);
+	return LDTP_ERROR_SUCCESS;
 }
 
 static LDTPErrorCode
 verify_partial_cell_value (LDTPClientContext* cctxt)
 {
 	char *text;
-	long row, column, n_rows, n_cols;
+	long row = 0, column = 0;
+	long n_rows, n_cols;
 	GSList *l;
 	LDTPErrorCode error;
 	int  child_type;
@@ -1256,13 +1400,13 @@ verify_partial_cell_value (LDTPClientContext* cctxt)
  
 	if (n_rows < row) {
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_ACTUAL_ROW_COUNT_LESS_THAN_GIVEN_ROW_COUNT);
+		error = LDTP_ERROR_ACTUAL_ROW_COUNT_LESS_THAN_GIVEN_ROW_COUNT;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 		return error;
 	}
 	if (n_cols < column) {
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_ACTUAL_COLUMN_COUNT_LESS_THAN_GIVEN_COLUMN_COUNT);
+		error = LDTP_ERROR_ACTUAL_COLUMN_COUNT_LESS_THAN_GIVEN_COLUMN_COUNT;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 		return error;
 	}  
@@ -1322,7 +1466,7 @@ verify_partial_cell_value (LDTPClientContext* cctxt)
 	Accessible_unref (table);
 	Accessible_unref (child_object);
 	cctxt->gui_handle->handle = object;
-	error =  (LDTP_ERROR_VERIFY_TABLE_CELL_PARTIAL_MATCH_FAILED);
+	error = LDTP_ERROR_VERIFY_TABLE_CELL_PARTIAL_MATCH_FAILED;
 	log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 	return error;
 }
@@ -1332,12 +1476,13 @@ get_row_index (Accessible *object, GSList **l, FILE *log_fp)
 {
 	long i, j;
 	long n_rows, n_cols;
+	long child_count;
 	char *name;
 	char *row_text;
 	LDTPErrorCode error;
 	Accessible *cell;
 	AccessibleTable *table;
-	char msg[256];
+	char msg [256];
 
 	table = Accessible_getTable (object);
 	n_rows = AccessibleTable_getNRows (table);
@@ -1347,28 +1492,31 @@ get_row_index (Accessible *object, GSList **l, FILE *log_fp)
 
 	for (i = 0; i < n_rows; i++) {
 		for (j = 0; j < n_cols; j++) {
-			long child_count;
 			cell = AccessibleTable_getAccessibleAt (table, i, j);
 			child_count = Accessible_getChildCount (cell);
+			g_print ("i=%ld j=%ld child_count=%ld\n", i, j, child_count);
 			if (child_count > 0) {
 				long k;
 				Accessible *child;
 				for (k = 0; k < child_count; k++) {
 					child = Accessible_getChildAtIndex (cell, k);
+					if (!child)
+						continue;
 					name = Accessible_getName (child);
+					if (name)
+						g_print ("Table cell name: %s\n", name);
 					if (g_utf8_collate (name, row_text) == 0) {
 						SPI_freeString (name);
 						Accessible_unref (child);
 						Accessible_unref (cell);
 						Accessible_unref (table);
 						*l = g_slist_prepend (*l, g_strdup_printf ("%ld", i));
-						return  (LDTP_ERROR_SUCCESS);
+						return LDTP_ERROR_SUCCESS;
 					} // if
 					SPI_freeString (name);
 					Accessible_unref (child);
 				} // for
-			} // if
-			else {
+			} else {
 				name = Accessible_getName (cell);
 				g_print ("Name = %s, Row Text = %s\n", name, row_text);
 				if (g_utf8_collate (name, row_text) == 0) {
@@ -1376,7 +1524,7 @@ get_row_index (Accessible *object, GSList **l, FILE *log_fp)
 					Accessible_unref (cell);
 					Accessible_unref (table);
 					*l = g_slist_prepend (*l, g_strdup_printf ("%ld", i));
-					return  (LDTP_ERROR_SUCCESS);
+					return LDTP_ERROR_SUCCESS;
 				}
 				SPI_freeString (name);
 			}
@@ -1388,7 +1536,7 @@ get_row_index (Accessible *object, GSList **l, FILE *log_fp)
 	g_sprintf (msg, "Unable to find %s in table", row_text);
 	*l = g_slist_prepend (*l, g_strdup_printf ("%ld", -1L));
 	log_msg (LDTP_LOG_CAUSE, msg, log_fp);
-	error =  (LDTP_ERROR_UNABLE_TO_GET_ROW_INDEX);
+	error = LDTP_ERROR_UNABLE_TO_GET_ROW_INDEX;
 	log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 	return error;
 }
@@ -1414,7 +1562,7 @@ single_click_row (Accessible *object, char *row_text, FILE *log_fp)
 			component = Accessible_getComponent (cell);
 		else {
 			Accessible_unref (cell);
-			error =  (LDTP_ERROR_OBJ_NOT_COMPONENT_TYPE);
+			error = LDTP_ERROR_OBJ_NOT_COMPONENT_TYPE;
 			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 			return error;
 		}
@@ -1429,20 +1577,20 @@ single_click_row (Accessible *object, char *row_text, FILE *log_fp)
 		Accessible_unref (cell);
 		Accessible_unref (component);
 		if (!flag) {
-			error =  (LDTP_ERROR_UNABLE_TO_GRAB_FOCUS);
+			error = LDTP_ERROR_UNABLE_TO_GRAB_FOCUS;
 			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 			return error;
 		}
 		flag = SPI_generateMouseEvent (x, y, "b1c");
 		if (flag)
-			return  (LDTP_ERROR_SUCCESS);
+			return LDTP_ERROR_SUCCESS;
 		else {
-			error =  (LDTP_ERROR_CLICK_FAILED);
+			error = LDTP_ERROR_CLICK_FAILED;
 			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 			return error;
 		}
 	}
-	error =  (LDTP_ERROR_UNABLE_TO_GET_CELL_HANDLE_FAILED);
+	error = LDTP_ERROR_UNABLE_TO_GET_CELL_HANDLE_FAILED;
 	log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 	return error;
 }
@@ -1584,8 +1732,19 @@ table_main (LDTPClientContext* cctxt, int command)
 				     cctxt->log_fp);
 		break;
 	case LDTP_CMD_KBDENTER:
+        case LDTP_CMD_MOUSEMOVE:
 		error = device_main (cctxt, command);
 		break;
+	case LDTP_CMD_RIGHTCLICK:
+		error = right_click (cctxt->gui_handle->handle, cctxt->req->arg_list, cctxt->log_fp);
+		break;
+	case LDTP_CMD_GETOBJECTSIZE:
+		cctxt->resp->data_len = 0;
+		cctxt->resp->data = get_size (cctxt->gui_handle->handle, &error);
+		if (cctxt->resp->data) {
+			cctxt->resp->data_len = g_utf8_strlen (cctxt->resp->data, -1);
+		}
+		break;
 	default:
 		error = LDTP_ERROR_COMMAND_NOT_IMPLEMENTED;
 	}
