@@ -3,9 +3,10 @@
  * Linux Desktop Testing Project http://ldtp.freedesktop.org
  *
  * Author:
- *    Nagappan A <nagappan@gmail.com>
+ *    Nagappan Alagappan <nagappan@gmail.com>
  *
  * Copyright 2004 - 2006 Novell, Inc.
+ * Copyright 2007 - 2008 Nagappan Alagappan
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -30,6 +31,8 @@
 #include "ldtp-command.h"
 #include "ldtp-gui-comp.h"
 
+extern gboolean ldtp_debug;
+
 static LDTPErrorCode
 is_text_state_enabled (Accessible *object, FILE *log_fp)
 {
@@ -50,17 +53,21 @@ set_text_value (Accessible *object, char *text, FILE *log_fp)
 {
 	LDTPErrorCode error;
 
-	if (Accessible_isEditableText (object)) {
+	if (text && Accessible_isEditableText (object)) {
 		SPIBoolean flag = FALSE;
+		AccessibleEditableText *editable_text;
 		if (Accessible_isComponent (object)) {
 			AccessibleComponent *accessible_component;
 			accessible_component = Accessible_getComponent (object);
 			flag = AccessibleComponent_grabFocus (accessible_component);
 			Accessible_unref (accessible_component);
 		}
-		AccessibleEditableText *editableText = Accessible_getEditableText (object);
-		flag = AccessibleEditableText_setTextContents (editableText, text);
-		Accessible_unref (editableText);
+		editable_text = Accessible_getEditableText (object);
+		if (editable_text) {
+			g_print ("Text: %s\n", text);
+			flag = AccessibleEditableText_setTextContents (editable_text, text);
+			Accessible_unref (editable_text);
+		}
 		if (flag == FALSE) {
 			g_print ("%s - %d\n", __FILE__, __LINE__);
 			error = LDTP_ERROR_UNABLE_TO_SET_TEXT;
@@ -301,7 +308,7 @@ get_text_value (Accessible *object)
 		char *tmp = NULL;
 		tmp = AccessibleText_getText (accessibleText, 0, LONG_MAX);
 		g_print ("Text: %s\n", tmp);
-		text = strdup (tmp);
+		text = g_strdup (tmp);
 		SPI_freeString (tmp);
 		Accessible_unref (accessibleText);
 	}
@@ -839,14 +846,16 @@ right_click (Accessible *object)
 		AccessibleComponent_getExtents (accessible_component,
 						&x, &y, &height, &width,
 						SPI_COORD_TYPE_WINDOW);
-		g_print ("X = %ld, Y = %ld, Width = %ld, Height = %ld\n",
-			 x, y, width, height);
-		g_print ("X = %ld, Y = %ld, Width / 2 = %ld, Height / 2 = %ld\n",
-			 x, y, width / 2, height / 2);
+		if (ldtp_debug) {
+			g_print ("X = %ld, Y = %ld, Width = %ld, Height = %ld\n",
+				 x, y, width, height);
+			g_print ("X = %ld, Y = %ld, Width / 2 = %ld, Height / 2 = %ld\n",
+				 x, y, (x + width) / 2, (y + height) / 2);
+		}
 		flag = AccessibleComponent_grabFocus (accessible_component);
 		Accessible_unref (accessible_component);
 	}
-	if (flag && SPI_generateMouseEvent (x + width / 2, y + height / 2, "b3c"))
+	if (flag && SPI_generateMouseEvent ((x + width) / 2, (y + height) / 2, "b3c"))
 		return LDTP_ERROR_SUCCESS;
 	else
 		return LDTP_ERROR_RIGHT_CLICK_FAILED;
@@ -1044,15 +1053,13 @@ text_main (LDTPClientContext* cctxt, int command)
                         error = device_main (cctxt, command);
                 break;
         case LDTP_CMD_MOUSEMOVE:
+	case LDTP_CMD_KBDENTER:
                 error = device_main (cctxt, command);
                 break;
-	case LDTP_CMD_KBDENTER:
-		error = device_main (cctxt, command);
-		break;
-	case LDTP_CMD_SETCURSOR:
+	case LDTP_CMD_SETCURSORPOSITION:
 		error = set_cursor_position (cctxt);
 		break;
-	case LDTP_CMD_GETCURSOR: {
+	case LDTP_CMD_GETCURSORPOSITION: {
 		glong position;
 		position = get_cursor_position (cctxt->gui_handle->handle);
 		if (position == -1)
