@@ -6,6 +6,7 @@
  *    Poornima Nayak <pnayak@novell.com>
  *
  * Copyright 2004 - 2006 Novell, Inc.
+ * Copyright 2007 - 2008 Nagappan Alagappan
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -30,6 +31,8 @@
 #include "ldtp-command.h"
 #include "ldtp-gui-comp.h"
 
+extern gboolean ldtp_debug;
+
 static gboolean
 get_state (Accessible *object)
 {
@@ -80,8 +83,10 @@ toggle (Accessible *object, FILE *log_fp)
 		return error;
 	}
 	action = Accessible_getAction (object);
-	flag = AccessibleAction_doAction (action, 0);
-	Accessible_unref (action);
+	if (action) {
+		flag = AccessibleAction_doAction (action, 0);
+		Accessible_unref (action);
+	}
 	if (flag == TRUE)
 		return LDTP_ERROR_SUCCESS;
 	else {
@@ -92,6 +97,32 @@ toggle (Accessible *object, FILE *log_fp)
 	}
 } 
 
+static LDTPErrorCode
+right_click (Accessible *object)
+{
+	gboolean flag = FALSE;
+	long x, y, height, width;
+	if (Accessible_isComponent (object)) {
+		AccessibleComponent *accessible_component;
+		accessible_component = Accessible_getComponent (object);
+		AccessibleComponent_getExtents (accessible_component,
+						&x, &y, &height, &width,
+						SPI_COORD_TYPE_WINDOW);
+		if (ldtp_debug) {
+			g_print ("X = %ld, Y = %ld, Width = %ld, Height = %ld\n",
+				 x, y, width, height);
+			g_print ("X = %ld, Y = %ld, Width / 2 = %ld, Height / 2 = %ld\n",
+				 x, y, x + width / 2, y + height / 2);
+		}
+		flag = AccessibleComponent_grabFocus (accessible_component);
+		Accessible_unref (accessible_component);
+	}
+	if (flag && SPI_generateMouseEvent ((x + width) / 2, (y + height) / 2, "b3c"))
+		return LDTP_ERROR_SUCCESS;
+	else
+		return LDTP_ERROR_RIGHT_CLICK_FAILED;
+}
+
 static LDTPErrorCode
 press (Accessible *object, FILE *log_fp)
 {
@@ -147,6 +178,9 @@ toggle_button_main (LDTPClientContext* cctxt, int command)
         case LDTP_CMD_GRABFOCUS:
                 error = grab_focus (cctxt->gui_handle->handle, cctxt->log_fp);
                 break;
+	case LDTP_CMD_RIGHTCLICK:
+		error = right_click (cctxt->gui_handle->handle);
+		break;
 	case LDTP_CMD_PRESS:
 		error = press (cctxt->gui_handle->handle, cctxt->log_fp);
 		break;
@@ -163,6 +197,13 @@ toggle_button_main (LDTPClientContext* cctxt, int command)
 	case LDTP_CMD_KBDENTER:
 		error = device_main (cctxt, command);
 		break;
+	case LDTP_CMD_GETOBJECTSIZE:
+		cctxt->resp->data_len = 0;
+		cctxt->resp->data = get_size (cctxt->gui_handle->handle, &error);
+		if (cctxt->resp->data) {
+			cctxt->resp->data_len = g_utf8_strlen (cctxt->resp->data, -1);
+		}
+		break;
 	default:
 		error = LDTP_ERROR_COMMAND_NOT_IMPLEMENTED;
 	}
