@@ -3,9 +3,10 @@
  * Linux Desktop Testing Project http://ldtp.freedesktop.org
  *
  * Author:
- *    Nagappan A <nagappan@gmail.com>
+ *    Nagappan Alagappan <nagappan@gmail.com>
  *
  * Copyright 2004 - 2006 Novell, Inc.
+ * Copyright 2007 - 2008 Nagappan Alagappan
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -56,7 +57,7 @@ sort_column_index (Accessible *object, int col, FILE *log_fp)
 		g_sprintf (msg, "Column %d doesnot exist in table", col);
 		log_msg (LDTP_LOG_CAUSE, msg, log_fp);
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_INVALID_COLUMN_INDEX_TO_SORT);
+		error = LDTP_ERROR_INVALID_COLUMN_INDEX_TO_SORT;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
@@ -82,11 +83,11 @@ sort_column_index (Accessible *object, int col, FILE *log_fp)
 	SPI_freeString (name);
 
 	if (flag == FALSE) { 
-		error =  (LDTP_ERROR_UNABLE_TO_SORT);
+		error = LDTP_ERROR_UNABLE_TO_SORT;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
-	return  (LDTP_ERROR_SUCCESS);
+	return LDTP_ERROR_SUCCESS;
 }
 
 /*
@@ -124,7 +125,7 @@ sort_column (Accessible *object, char *row_text, FILE *log_fp)
   
 	if (i == n_cols) {	  
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_COLUMN_DOES_NOT_EXIST);
+		error = LDTP_ERROR_COLUMN_DOES_NOT_EXIST;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
@@ -147,11 +148,11 @@ sort_column (Accessible *object, char *row_text, FILE *log_fp)
 	SPI_freeString (name);
 
 	if (flag == FALSE) { 
-		error =  (LDTP_ERROR_UNABLE_TO_SORT);
+		error = LDTP_ERROR_UNABLE_TO_SORT;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
-	return  (LDTP_ERROR_SUCCESS);
+	return LDTP_ERROR_SUCCESS;
 }  
 
 static LDTPErrorCode
@@ -187,14 +188,14 @@ get_cell_value (LDTPClientContext *cctxt)
 	if (n_rows < row) {
 		g_print ("Row cout: %ld - Actual: %ld\n", row, n_rows);
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_ACTUAL_ROW_COUNT_LESS_THAN_GIVEN_ROW_COUNT);
+		error = LDTP_ERROR_ACTUAL_ROW_COUNT_LESS_THAN_GIVEN_ROW_COUNT;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 		return error;
 	}
 	if (n_cols < column) {
 		g_print ("Column cout: %ld - Actual: %ld\n", column, n_cols);
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_ACTUAL_COLUMN_COUNT_LESS_THAN_GIVEN_COLUMN_COUNT);
+		error = LDTP_ERROR_ACTUAL_COLUMN_COUNT_LESS_THAN_GIVEN_COLUMN_COUNT;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 		return error;
 	}
@@ -259,8 +260,7 @@ get_cell_value (LDTPClientContext *cctxt)
 				Accessible_unref (child_object);
 				cctxt->gui_handle->handle = object;
 				return error;
-			}
-			else {
+			} else {
 				AccessibleStateSet* stateset;
 				stateset = Accessible_getStateSet (child_object);
 				if (AccessibleStateSet_contains (stateset, SPI_STATE_CHECKED))
@@ -272,7 +272,7 @@ get_cell_value (LDTPClientContext *cctxt)
 				Accessible_unref (table);
 				Accessible_unref (child_object);
 				cctxt->gui_handle->handle = object;
-				return  (LDTP_ERROR_SUCCESS);
+				return LDTP_ERROR_SUCCESS;
 			}
 		}
 	} //if
@@ -280,7 +280,7 @@ get_cell_value (LDTPClientContext *cctxt)
 	Accessible_unref (table);
 	Accessible_unref (child_object);
 	cctxt->gui_handle->handle = object;
-	error =  (LDTP_ERROR_GET_TREE_TABLE_CELL_FAILED);
+	error = LDTP_ERROR_GET_TREE_TABLE_CELL_FAILED;
 	log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 	return error;
 }
@@ -317,13 +317,13 @@ verify_partial_cell_value (LDTPClientContext* cctxt)
  
 	if (n_rows < row) {
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_ACTUAL_ROW_COUNT_LESS_THAN_GIVEN_ROW_COUNT);
+		error = LDTP_ERROR_ACTUAL_ROW_COUNT_LESS_THAN_GIVEN_ROW_COUNT;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 		return error;
 	}
 	if (n_cols < column) {
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_ACTUAL_COLUMN_COUNT_LESS_THAN_GIVEN_COLUMN_COUNT);
+		error = LDTP_ERROR_ACTUAL_COLUMN_COUNT_LESS_THAN_GIVEN_COLUMN_COUNT;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 		return error;
 	}  
@@ -377,7 +377,7 @@ verify_partial_cell_value (LDTPClientContext* cctxt)
 	Accessible_unref (table);
 	Accessible_unref (child_object);
 	cctxt->gui_handle->handle = object;
-	error =  (LDTP_ERROR_VERIFY_TABLE_CELL_PARTIAL_MATCH_FAILED);
+	error = LDTP_ERROR_VERIFY_TABLE_CELL_PARTIAL_MATCH_FAILED;
 	log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 	return error;
 }
@@ -489,7 +489,7 @@ select_row (Accessible *object, GSList *l, FILE *log_fp)
 								Accessible_unref (accessible_component);
 								Accessible_unref (child);
 								Accessible_unref (cell);
-								error =  (LDTP_ERROR_UNABLE_TO_GRAB_FOCUS);
+								error = LDTP_ERROR_UNABLE_TO_GRAB_FOCUS;
 								log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 								SPI_freeString(name);
 								return error;
@@ -522,7 +522,7 @@ select_row (Accessible *object, GSList *l, FILE *log_fp)
 						if (!AccessibleComponent_grabFocus (accessible_component)) {
 							Accessible_unref (accessible_component);
 							Accessible_unref (cell);
-							error =  (LDTP_ERROR_UNABLE_TO_GRAB_FOCUS);
+							error = LDTP_ERROR_UNABLE_TO_GRAB_FOCUS;
 							log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 							return error;
 						}
@@ -542,11 +542,11 @@ select_row (Accessible *object, GSList *l, FILE *log_fp)
   
 	Accessible_unref (table);
 	if (flag == FALSE) {
-		error =  (LDTP_ERROR_UNABLE_TO_SELECT_ROW);
+		error = LDTP_ERROR_UNABLE_TO_SELECT_ROW;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
-	return  (LDTP_ERROR_SUCCESS);
+	return LDTP_ERROR_SUCCESS;
 }
 
 static LDTPErrorCode
@@ -590,7 +590,7 @@ select_row_partial_match (Accessible *object, char *row_text, FILE *log_fp)
 						if (flag == FALSE) {
 							Accessible_unref (child);
 							Accessible_unref (cell);
-							error =  (LDTP_ERROR_UNABLE_TO_GRAB_FOCUS);
+							error = LDTP_ERROR_UNABLE_TO_GRAB_FOCUS;
 							log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 							return error;
 						}
@@ -615,11 +615,11 @@ select_row_partial_match (Accessible *object, char *row_text, FILE *log_fp)
 		char msg[256];
 		g_sprintf (msg, "Unable to find %s in table", row_text);
 		log_msg (LDTP_LOG_CAUSE, msg, log_fp);
-		error =  (LDTP_ERROR_UNABLE_TO_SELECT_ROW);
+		error = LDTP_ERROR_UNABLE_TO_SELECT_ROW;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
-	return  (LDTP_ERROR_SUCCESS);
+	return LDTP_ERROR_SUCCESS;
 }
 
 static LDTPErrorCode
@@ -656,7 +656,7 @@ get_current_row_index (Accessible *object, GSList **l, FILE *log_fp)
 						Accessible_unref (cell);
 						Accessible_unref (table);
 						*l = g_slist_prepend (*l, g_strdup_printf ("%ld", i));
-						return  (LDTP_ERROR_SUCCESS);
+						return LDTP_ERROR_SUCCESS;
 					} // if
 					SPI_freeString (name);
 					Accessible_unref (child);
@@ -670,7 +670,7 @@ get_current_row_index (Accessible *object, GSList **l, FILE *log_fp)
 					Accessible_unref (cell);
 					Accessible_unref (table);
 					*l = g_slist_prepend (*l, g_strdup_printf ("%ld", i));
-					return  (LDTP_ERROR_SUCCESS);
+					return LDTP_ERROR_SUCCESS;
 				}
 				SPI_freeString (name);
 			}
@@ -681,7 +681,7 @@ get_current_row_index (Accessible *object, GSList **l, FILE *log_fp)
 	Accessible_unref (table);
 	g_sprintf (msg, "Unable to find %s in table", row_text);
 	log_msg (LDTP_LOG_CAUSE, msg, log_fp);
-	error =  (LDTP_ERROR_UNABLE_TO_GET_ROW_INDEX);
+	error = LDTP_ERROR_UNABLE_TO_GET_ROW_INDEX;
 	log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 	return error;
 }
@@ -701,7 +701,7 @@ double_click (Accessible *object, FILE *log_fp)
 		component = Accessible_getComponent (object);
 	else {
 		Accessible_unref (object);
-		error =  (LDTP_ERROR_OBJ_NOT_COMPONENT_TYPE);
+		error = LDTP_ERROR_OBJ_NOT_COMPONENT_TYPE;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
@@ -715,15 +715,15 @@ double_click (Accessible *object, FILE *log_fp)
 	flag = AccessibleComponent_grabFocus (component);
 	Accessible_unref (component);
 	if (!flag) {
-		error =  (LDTP_ERROR_UNABLE_TO_GRAB_FOCUS);
+		error = LDTP_ERROR_UNABLE_TO_GRAB_FOCUS;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
 	flag = SPI_generateMouseEvent (x, y, "b1d");
 	if (flag)
-		return  (LDTP_ERROR_SUCCESS);
+		return LDTP_ERROR_SUCCESS;
 	else {
-		error =  (LDTP_ERROR_DOUBLE_CLICK_FAILED);
+		error = LDTP_ERROR_DOUBLE_CLICK_FAILED;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
@@ -747,7 +747,7 @@ double_click_row (Accessible *object, char *row_text, FILE *log_fp)
 			component = Accessible_getComponent (cell);
 		else {
 			Accessible_unref (cell);
-			error =  (LDTP_ERROR_OBJ_NOT_COMPONENT_TYPE);
+			error = LDTP_ERROR_OBJ_NOT_COMPONENT_TYPE;
 			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 			return error;
 		}
@@ -762,20 +762,20 @@ double_click_row (Accessible *object, char *row_text, FILE *log_fp)
 		Accessible_unref (cell);
 		Accessible_unref (component);
 		if (!flag) {
-			error =  (LDTP_ERROR_UNABLE_TO_GRAB_FOCUS);
+			error = LDTP_ERROR_UNABLE_TO_GRAB_FOCUS;
 			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 			return error;
 		}
 		flag = SPI_generateMouseEvent (x, y, "b1d");
 		if (flag)
-			return  (LDTP_ERROR_SUCCESS);
+			return LDTP_ERROR_SUCCESS;
 		else {
-			error =  (LDTP_ERROR_DOUBLE_CLICK_FAILED);
+			error = LDTP_ERROR_DOUBLE_CLICK_FAILED;
 			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 			return error;
 		}
 	}
-	error =  (LDTP_ERROR_UNABLE_TO_GET_CELL_HANDLE_FAILED);
+	error = LDTP_ERROR_UNABLE_TO_GET_CELL_HANDLE_FAILED;
 	log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 	return error;
 }
@@ -800,7 +800,7 @@ get_row_index (Accessible *object, long index, FILE *log_fp)
 		g_sprintf (msg, "Row %ld doesnot exist in table", index);
 		log_msg (LDTP_LOG_CAUSE, msg, log_fp);
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_ROW_DOES_NOT_EXIST);
+		error = LDTP_ERROR_ROW_DOES_NOT_EXIST;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return NULL;
 	}
@@ -831,7 +831,7 @@ double_click_row_index (Accessible *object, long index, FILE *log_fp)
 			component = Accessible_getComponent (cell);
 		else {
 			Accessible_unref (cell);
-			error =  (LDTP_ERROR_OBJ_NOT_COMPONENT_TYPE);
+			error = LDTP_ERROR_OBJ_NOT_COMPONENT_TYPE;
 			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 			return error;
 		}
@@ -846,15 +846,15 @@ double_click_row_index (Accessible *object, long index, FILE *log_fp)
 		Accessible_unref (cell);
 		Accessible_unref (component);
 		if (!flag) {
-			error =  (LDTP_ERROR_UNABLE_TO_GRAB_FOCUS);
+			error = LDTP_ERROR_UNABLE_TO_GRAB_FOCUS;
 			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 			return error;
 		}
 		flag = SPI_generateMouseEvent (x, y, "b1d");
 		if (flag)
-			return  (LDTP_ERROR_SUCCESS);
+			return LDTP_ERROR_SUCCESS;
 		else {
-			error =  (LDTP_ERROR_DOUBLE_CLICK_FAILED);
+			error = LDTP_ERROR_DOUBLE_CLICK_FAILED;
 			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 			return error;
 		}
@@ -879,8 +879,8 @@ select_row_index (Accessible *object, long row, FILE *log_fp)
 		Accessible_unref (accessible_component);
 		Accessible_unref (cell);
 		if (flag)
-			return  (LDTP_ERROR_SUCCESS);
-		error =  (LDTP_ERROR_UNABLE_TO_GRAB_FOCUS);
+			return LDTP_ERROR_SUCCESS;
+		error = LDTP_ERROR_UNABLE_TO_GRAB_FOCUS;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
@@ -888,7 +888,7 @@ select_row_index (Accessible *object, long row, FILE *log_fp)
 		if (cell)
 			Accessible_unref (cell);
 		log_msg (LDTP_LOG_CAUSE, "Object is not accessible component", log_fp);
-		error =  (LDTP_ERROR_OBJ_NOT_COMPONENT_TYPE);
+		error = LDTP_ERROR_OBJ_NOT_COMPONENT_TYPE;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
@@ -919,9 +919,9 @@ select_last_row (Accessible *object, FILE *log_fp)
 		Accessible_unref (cell);
 		Accessible_unref (table);
 		if (flag == TRUE)
-			return  (LDTP_ERROR_SUCCESS);
+			return LDTP_ERROR_SUCCESS;
 		else {
-			error =  (LDTP_ERROR_UNABLE_TO_GRAB_FOCUS);
+			error = LDTP_ERROR_UNABLE_TO_GRAB_FOCUS;
 			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 			return error;
 		}
@@ -929,7 +929,7 @@ select_last_row (Accessible *object, FILE *log_fp)
 	else {
 		Accessible_unref (cell);
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_OBJ_NOT_COMPONENT_TYPE);
+		error = LDTP_ERROR_OBJ_NOT_COMPONENT_TYPE;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
@@ -1042,10 +1042,10 @@ right_click (Accessible *object, GSList *l, FILE *log_fp)
 		g_print ("x=%ld, y=%ld, width=%ld, height=%ld\n", x, y, width, height);
 		/*
 		if (SPI_generateMouseEvent (x + width / 2, y + height / 2, "b3c"))
-			return  (LDTP_ERROR_SUCCESS);
+			return LDTP_ERROR_SUCCESS;
 		*/
 		if (SPI_generateMouseEvent (x, y, "b3c"))
-			return  (LDTP_ERROR_SUCCESS);
+			return LDTP_ERROR_SUCCESS;
 	}
 	error = LDTP_ERROR_RIGHT_CLICK_FAILED;
 	log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
@@ -1065,7 +1065,7 @@ get_row_count (Accessible *object, GSList **l)
 
 	Accessible_unref(table);
 	*l = g_slist_prepend (*l, g_strdup_printf ("%ld", n_rows));
-	return  (LDTP_ERROR_SUCCESS);
+	return LDTP_ERROR_SUCCESS;
 }
 
 static LDTPErrorCode
@@ -1085,10 +1085,10 @@ does_row_exists (Accessible *object, char *row_text, FILE *log_fp)
 		if (n_rows != 0) {
 			log_msg (LDTP_LOG_INFO, "Rows available in Table cell", log_fp);
 			Accessible_unref (table);
-			return  (LDTP_ERROR_SUCCESS);
+			return LDTP_ERROR_SUCCESS;
 		}
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_ROW_DOES_NOT_EXIST);
+		error = LDTP_ERROR_ROW_DOES_NOT_EXIST;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 		return error;
 	}
@@ -1133,9 +1133,9 @@ does_row_exists (Accessible *object, char *row_text, FILE *log_fp)
 		} // for i
 		Accessible_unref (table);
 		if (flag == TRUE)
-			return  (LDTP_ERROR_SUCCESS);
+			return LDTP_ERROR_SUCCESS;
 		else {
-			error =  (LDTP_ERROR_ROW_DOES_NOT_EXIST);
+			error = LDTP_ERROR_ROW_DOES_NOT_EXIST;
 			log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 			return error;
 		}
@@ -1173,13 +1173,13 @@ verify_cell_value (LDTPClientContext* cctxt)
  
 	if (n_rows < row) {
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_ACTUAL_ROW_COUNT_LESS_THAN_GIVEN_ROW_COUNT);
+		error = LDTP_ERROR_ACTUAL_ROW_COUNT_LESS_THAN_GIVEN_ROW_COUNT;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 		return error;
 	}
 	if (n_cols < column) {
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_ACTUAL_COLUMN_COUNT_LESS_THAN_GIVEN_COLUMN_COUNT);
+		error = LDTP_ERROR_ACTUAL_COLUMN_COUNT_LESS_THAN_GIVEN_COLUMN_COUNT;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 		return error;
 	}  
@@ -1222,7 +1222,8 @@ verify_cell_value (LDTPClientContext* cctxt)
 					cctxt->gui_handle->handle = object;
 					return error;
 				}
-				if (g_ascii_strcasecmp (name, "edit") == 0) {
+				if (g_ascii_strcasecmp (name, "edit") == 0 ||
+				    g_ascii_strcasecmp (name, "activate") == 0) {
 					GSList *l = g_slist_last (cctxt->req->arg_list);
 
 					cctxt->req->arg_list = g_slist_prepend (cctxt->req->arg_list, l);
@@ -1268,7 +1269,7 @@ verify_cell_value (LDTPClientContext* cctxt)
 	Accessible_unref (table);
 	Accessible_unref (child_object);
 	cctxt->gui_handle->handle = object;
-	error =  (LDTP_ERROR_VERIFY_TABLE_CELL_FAILED);
+	error = LDTP_ERROR_VERIFY_TABLE_CELL_FAILED;
 	log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 	return error;
 }  
@@ -1301,13 +1302,13 @@ set_cell_value (LDTPClientContext* cctxt)
 
 	if (n_rows < row) {
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_ACTUAL_ROW_COUNT_LESS_THAN_GIVEN_ROW_COUNT);
+		error = LDTP_ERROR_ACTUAL_ROW_COUNT_LESS_THAN_GIVEN_ROW_COUNT;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 		return error;
 	}
 	if (n_cols < column) {
 		Accessible_unref (table);
-		error =  (LDTP_ERROR_ACTUAL_COLUMN_COUNT_LESS_THAN_GIVEN_COLUMN_COUNT);
+		error = LDTP_ERROR_ACTUAL_COLUMN_COUNT_LESS_THAN_GIVEN_COLUMN_COUNT;
 		log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 		return error;
 	}  
@@ -1315,7 +1316,7 @@ set_cell_value (LDTPClientContext* cctxt)
 	g_print ("Tree table - number of rows:%ld\tColumn: %ld\n", n_rows, n_cols);
 	cell = AccessibleTable_getAccessibleAt (table, row, column);
 	cctxt->gui_handle->handle = cell;
-	if (Accessible_isComponent (cell)) {
+	if (cell && Accessible_isComponent (cell)) {
 		long j;
 		action = Accessible_getAction (cell);
 		n_actions = AccessibleAction_getNActions (action);
@@ -1323,12 +1324,8 @@ set_cell_value (LDTPClientContext* cctxt)
 			name = AccessibleAction_getName (action, j);
 			/* Handled at present only Toggle & Text type of cell */
 			if (g_ascii_strcasecmp (name, "toggle") == 0) {
-				GSList *l = g_slist_last (cctxt->req->arg_list);
-
-				cctxt->req->arg_list = g_slist_prepend (cctxt->req->arg_list, l);
 				g_print ("Table cell is of type 'Toggle'\n");
-				error = toggle_button_main (cctxt, LDTP_CMD_CLICK); 
-				cctxt->req->arg_list = g_slist_remove (cctxt->req->arg_list, l->data);
+				error = toggle_button_main (cctxt, LDTP_CMD_CLICK);
 				SPI_freeString (name);
 				Accessible_unref (action);
 				Accessible_unref (cell);
@@ -1337,12 +1334,14 @@ set_cell_value (LDTPClientContext* cctxt)
 				return error;
 			}
 			if (g_ascii_strcasecmp (name, "edit") == 0) {
-				GSList *l = g_slist_last (cctxt->req->arg_list);
+				GSList *tmp_list = g_slist_last (cctxt->req->arg_list);
 
-				cctxt->req->arg_list = g_slist_prepend (cctxt->req->arg_list, l);
+				cctxt->req->arg_list = g_slist_prepend (cctxt->req->arg_list,
+									tmp_list);
 				g_print ("Table cell is text box\n");
 				error = text_main (cctxt, LDTP_CMD_SETTEXTVALUE);
-				cctxt->req->arg_list = g_slist_remove (cctxt->req->arg_list, l->data);
+				cctxt->req->arg_list = g_slist_remove (cctxt->req->arg_list,
+								       tmp_list->data);
 				SPI_freeString (name);
 				Accessible_unref (action);
 				Accessible_unref (cell);
@@ -1354,10 +1353,11 @@ set_cell_value (LDTPClientContext* cctxt)
 		} //for
 		Accessible_unref (action);
 	} //if
-	Accessible_unref (cell);
+	if (cell)
+		Accessible_unref (cell);
 	Accessible_unref (table);
 	cctxt->gui_handle->handle = object;
-	error =  (LDTP_ERROR_SET_TABLE_CELL_FAILED);
+	error = LDTP_ERROR_SET_TABLE_CELL_FAILED;
 	log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), cctxt->log_fp);
 	return error;
 }  
@@ -1367,7 +1367,7 @@ do_action (Accessible *object, long row, long col, char* command, FILE *log_fp)
 {
 	long j;
 	long n_rows, n_cols;
-	char msg[256];
+	char msg [256];
 	char *name = NULL;
 	LDTPErrorCode error;
 	Accessible *cell;
@@ -1403,14 +1403,14 @@ do_action (Accessible *object, long row, long col, char* command, FILE *log_fp)
 						if (AccessibleAction_doAction (childAction, j)) {
 							SPI_freeString (name);
 							AccessibleAction_unref (childAction);
-							return  (LDTP_ERROR_SUCCESS);
+							return LDTP_ERROR_SUCCESS;
 						}
 						else {
 							AccessibleAction_unref (childAction);
 							g_sprintf (msg, "Failed to perform %s action on object!!", name);
 							log_msg (LDTP_LOG_CAUSE, msg, log_fp);
 							SPI_freeString (name);
-							error =  (LDTP_ERROR_UNABLE_TO_PERFORM_ACTION);
+							error = LDTP_ERROR_UNABLE_TO_PERFORM_ACTION;
 							log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 							return error;
 						}
@@ -1430,14 +1430,14 @@ do_action (Accessible *object, long row, long col, char* command, FILE *log_fp)
 					if (AccessibleAction_doAction (childAction, j)) {
 						SPI_freeString (name);
 						Accessible_unref (childAction);
-						return  (LDTP_ERROR_SUCCESS);
+						return LDTP_ERROR_SUCCESS;
 					}
 					else {
 						AccessibleAction_unref (childAction);
 						g_sprintf (msg, "Failed to perform %s action on object!!", name);
 						log_msg (LDTP_LOG_CAUSE, msg, log_fp);
 						SPI_freeString (name);
-						error =  (LDTP_ERROR_UNABLE_TO_PERFORM_ACTION);
+						error = LDTP_ERROR_UNABLE_TO_PERFORM_ACTION;
 						log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 						return error;
 					}
@@ -1448,7 +1448,7 @@ do_action (Accessible *object, long row, long col, char* command, FILE *log_fp)
 		}
 	}
 	Accessible_unref (cell);
-	error =  (LDTP_ERROR_UNABLE_TO_PERFORM_ACTION);
+	error = LDTP_ERROR_UNABLE_TO_PERFORM_ACTION;
 	log_msg (LDTP_LOG_CAUSE, ldtp_error_get_message (error), log_fp);
 	return error;
 }
@@ -1572,7 +1572,7 @@ tree_table_main (LDTPClientContext* cctxt, int command)
 					   row, col, "toggle", cctxt->log_fp);
 		else {
 			log_msg (LDTP_LOG_WARNING, "Row already checked", cctxt->log_fp);
-			error =  (LDTP_ERROR_SUCCESS);
+			error = LDTP_ERROR_SUCCESS;
 		}
 		break;
 	}
@@ -1592,7 +1592,7 @@ tree_table_main (LDTPClientContext* cctxt, int command)
 					   row, col, "toggle", cctxt->log_fp);
 		else {
 			log_msg (LDTP_LOG_WARNING, "Row already unchecked", cctxt->log_fp);
-			error =  (LDTP_ERROR_SUCCESS);
+			error = LDTP_ERROR_SUCCESS;
 		}
 		break;
 	}
@@ -1608,9 +1608,9 @@ tree_table_main (LDTPClientContext* cctxt, int command)
 			}
 		}
 		if (get_status (cctxt->gui_handle->handle, row, col) == TRUE)
-			error =  (LDTP_ERROR_STATE_CHECKED);
+			error = LDTP_ERROR_STATE_CHECKED;
 		else
-			error =  (LDTP_ERROR_STATE_UNCHECKED);
+			error = LDTP_ERROR_STATE_UNCHECKED;
 		break;
 	}
 	case LDTP_CMD_VERIFYUNCHECKROW: {
@@ -1625,9 +1625,9 @@ tree_table_main (LDTPClientContext* cctxt, int command)
 			}
 		}
 		if (get_status (cctxt->gui_handle->handle, row, col) == TRUE)
-			error =  (LDTP_ERROR_STATE_CHECKED);
+			error = LDTP_ERROR_STATE_CHECKED;
 		else
-			error =  (LDTP_ERROR_STATE_UNCHECKED);
+			error = LDTP_ERROR_STATE_UNCHECKED;
 		break;
 	}
 	case LDTP_CMD_EXPANDTABLECELL: {
@@ -1659,6 +1659,13 @@ tree_table_main (LDTPClientContext* cctxt, int command)
 	case LDTP_CMD_KBDENTER:
 		error = device_main (cctxt, command);
 		break;
+	case LDTP_CMD_GETOBJECTSIZE:
+		cctxt->resp->data_len = 0;
+		cctxt->resp->data = get_size (cctxt->gui_handle->handle, &error);
+		if (cctxt->resp->data) {
+			cctxt->resp->data_len = g_utf8_strlen (cctxt->resp->data, -1);
+		}
+		break;
 	default:
 		error = LDTP_ERROR_COMMAND_NOT_IMPLEMENTED;
 	}
